<?php

namespace Teknisa\CRM\Factory;

use Doctrine\ORM\EntityManager;

class AgrupamentoParceiroFactory {

    protected $connection;

    public function __construct(EntityManager $entityManager) {
        
        $this->connection = $entityManager->getConnection();
    }
    
    public function queryDeleteGroup($paramsQuery){
        try {
            $delete = "DELETE FROM CRM_AGRUPAMENTOPARCEIRO
                        WHERE NRAGRPARC = :NRAGRPARC
                        AND CRM_NRORGTRAB = :CRM_NRORGTRAB
                        AND NRORG = :NRORG";
            
            $std = $this->connection->prepare($delete);
            $std->execute($paramsQuery);
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function verifyGroup($paramsQuery){
        try {
            $query = "SELECT 
                         NRAGRPARC, 
                         NMPRINCIPALPARC,
                         NRINSCRICAOPARC
                       FROM 
                         CRM_AGRUPAMENTOPARCEIRO AGR
                         LEFT JOIN PARCNEGOCIO PARC
                           ON PARC.NRPARCNEGOCIO = AGR.NRPARCNEGOCIO
                        WHERE 
                          AGR.NRPARCNEGOCIO_REL = :NRPARCNEGOCIO_REL
                          AND AGR.CRM_NRORGTRAB = :CRM_NRORGTRAB
                          AND AGR.NRORG = :NRORG";
            
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            
            $resp = $std->fetchAll();
            
            if(!empty($resp)){
                return $resp[0];
            } else {
                return $resp;
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function agrupamentoEvolSave($paramsQuery){
        try {
            $query = "INSERT INTO 
                      EVOLCONT(NRCONTATO, DTHREVOLCONT, CDTIPOCONT, IDSENEVOCONT, IDTPEVOLCONT, CDOPEREVOL, CDMOTIEVOCON, NRORG, CRM_NRORGTRAB, DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, CDOPERULTATU, TXEVOLCONT, IDATIVO, NRSEQRESULEVOL, NRSEQEVOLCONT, NRSEQCONTATO, IDGERAFATURA, IDANEXOEVOLCONT, CDFILIAL)
                      VALUES(:NRCONTATO, :DTHREVOLCONT, :CDTIPOCONT, :IDSENEVOCONT, 'P', :CDOPEREVOL, :CDMOTIEVOCON, :NRORG, :CRM_NRORGTRAB, SYSDATE, SYSDATE, :NRORG, :CDOPEREVOL, :NRORG, :CDOPEREVOL, :TXEVOLCONT, 'S', :NRSEQRESULEVOL, :NRSEQEVOLCONT, :NRSEQCONTATO, 'NAO', 'N', :CDFILIAL)";
            
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
        } catch(\Exception $e) {
            $e->getMessage();
            throw new \Exception($e);
        }
    }
    
    public function searchNrContato($paramsQuery){
        try {
            $query = "SELECT 
                         NRCONTATO, 
                         NRSEQCONTATO 
                       FROM 
                         CONTATO
                        WHERE 
                          NRPARCNEGOCIO = :NRPARCNEGOCIO
                          AND NRORG = :NRORG";
            
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            
            $resp = $std->fetchAll();
            
            if(!empty($resp)){
                return $resp[0];
            } else {
                return array();
            }
        } catch(\Exception $e) {
            $e->getMessage();
        }
    }
    
    public function verifyParcGroupRel($paramsQuery){
        try {
            $query = "SELECT
                         AGR.NRAGRPARC, 
                         PARC.NRPARCNEGOCIO,
                         PARC.NMPRINCIPALPARC,
                         PARC.NRINSCRICAOPARC
                       FROM 
                         CRM_AGRUPAMENTOPARCEIRO AGR
                         LEFT JOIN PARCNEGOCIO PARC
                           ON PARC.NRPARCNEGOCIO = AGR.NRPARCNEGOCIO_REL
                        WHERE 
                          AGR.NRPARCNEGOCIO = :NRPARCNEGOCIO
                          AND AGR.CRM_NRORGTRAB = :CRM_NRORGTRAB
                          AND AGR.NRORG = :NRORG";
            
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            
            $resp = $std->fetchAll();
            
            if(!empty($resp)){
                return $resp;
            } else {
                return array();
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function searchParcnegocio($paramsQuery){
        try {
            $query = "SELECT
                        PARC.NRPARCNEGOCIO,
                        PARC.NMPRINCIPALPARC,
                        PARC.NRINSCRICAOPARC
                      FROM
                        PARCNEGOCIO PARC
                        LEFT JOIN CRM_STATUSPARC STAT
                          ON STAT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                            AND STAT.NRORG = PARC.NRORG
                            AND STAT.CRM_NRORGTRAB = :CRM_NRORGTRAB
                      WHERE 
                        PARC.NRPARCNEGOCIO = :NRPARCNEGOCIO
                        AND PARC.NRORG = :NRORG
                        AND STAT.STATUSPARC <> 'CLIENTE'";
            
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            
            $resp = $std->fetchAll();
            
            if(!empty($resp)){
                return $resp[0];
            } else {
                return array();
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getRelationshipPartnerStatus($paramsQuery){
        try {
            $query = "SELECT
                        NRPARCNEGOCIO,
                        NMPRINCIPALPARC,
                        NRINSCRICAOPARC
                      FROM
                        PARCNEGOCIO PARC
                      WHERE 
                        NRPARCNEGOCIO = :NRPARCNEGOCIO
                        AND AGR.NRORG = :NRORG";
            
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
            
            $resp = $std->fetchAll();
            
            if(!empty($resp)){
                return $resp[0];
            } else {
                return array();
            }
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function setRelationshipPartnerStatus($paramsQuery){
        try {
            $query = "UPDATE
                        CRM_STATUSPARC
                      SET
                        STATUSPARC = :STATUSPARC
                      WHERE
                        NRORG = :NRORG
                        AND CRM_NRORGTRAB = :CRM_NRORGTRAB
                        AND NRPARCNEGOCIO = :NRPARCNEGOCIO
                     ";
            
            $std = $this->connection->prepare($query);
            $std->execute($paramsQuery);
        } catch(\Exception $e) {
            throw $e;
        }
    }
}