<?php
namespace Teknisa\CRM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Util\NovoCodigo;
use Teknisa\CRM\Util\Util;
use \Doctrine\Common\Collections\Criteria;

class TicketBotFactory
{
    protected $entityManager;
    protected $environment;
    protected $mailInfo;
    /*Usado para armazenar mensagens que informarão os passos percorridos pela rotina*/
    protected $returnMessage = array();
    /**
     * Usado para armazenar as keys inseridas nas tabelas 
     *  NRPARCNEGOCIO       -> Solicitante/Interlocutor do ticket 
     *  NRPARCNEGOCIOPARENT -> Empresa do Solicitante/Interlocutor do ticket
     * 
    */
    protected $keys = array();
    
    public function __construct(EntityManager $entityManager, $environment, $operController, $zhFileUploader) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->operController = $operController;
        $this->zhFileUploader = $zhFileUploader;
    }
    
    public function saveTicketFromMailInfo($mailInfo){
        $this->returnMessage = [];
        $this->mailInfo = $mailInfo;
        $this->findMailInCRM();
        $this->createTicket();
        if(!empty($mailInfo['ATTACH']))
            $this->uploadAttachment();
        
        //Retorna um array com a mensagem mostrando os passos da rotina(Criação parceiro, Criação Ticket...) e com o nrseqcontato do ticket criado.
        return array('MESSAGE' => implode($this->returnMessage, ', '), 'TCTID' => empty($this->keys['NRSEQCONTATO']) ? '' : $this->keys['NRSEQCONTATO'], 'PARCNEGOCIO' => empty($this->keys['NRPARCNEGOCIO']) ? '' : $this->keys['NRPARCNEGOCIO']);
    }
    
    /**
     *  Busca o email que solicitou o ticket na base do produto, se não encontrar cria o parceiro.
     *  Procura nas tabelas COMUNICAPARC, CRM_INTERLOCUTOR e OPERADOR
     *  
    */
    private function findMailInCRM(){
        
        $comunicaparc = $this->findInEntity('\Teknisa\Model\Geral\Entities\Comunicaparc', 'cdcomunicaparc');
        $interlocutor = $this->findInEntity('\Teknisa\Model\Crm\Entities\CrmInterlocutor', 'emailinterloc');
        $operador     = $this->findInEntity('\Teknisa\Model\Geral\Entities\Operador', 'dsemailoper');
        
        $this->setParcAndFindParent($comunicaparc, $operador, $interlocutor);
        
        if(empty($this->keys['NRPARCNEGOCIO'])){
            $this->createParc();
        }
    }
    
    /**
     *  Busca o email na entidade passada como parâmetro
     * 
     *  @param string   $entityPath     Caminho da entidade.
     *  @param string   $mailField      Nome do campo de email da entidade.
     * 
     *  @return object                  Instância da Entidade, se encontrar o email. 
     *  
    */
    public function findInEntity($entityPath, $mailField){
        $email = $this->mailInfo['FROMMAIL'];
        
        $obj = $this->entityManager->getRepository($entityPath)->findOneBy(
            array(
                $mailField => $email,
                'nrorg' => $this->environment->getNrOrgTrab()
            )
        );
        
        if(is_object($obj)){
            //Pega o nome da entidade pelo caminho e grava que encontrou.
            $entityName = explode('\\', $entityPath);
            $this->returnMessage[] = 'Mail found in '.$entityName[count($entityName)-1];
            return $obj;
        }
    }
    
    public function createParc() {
        //PARCNEGOCIO    
        $nrparcnegocio = NovoCodigo::geraCodigo($this->environment->getNrOrgTrab(), "PARCNEGOCIO", 1, 10, false, false);
        $parcnegocio = new \Teknisa\Model\Geral\Entities\Parcnegocio();
        
        $parcnegocio->setNrparcnegocio($nrparcnegocio);
        $parcnegocio->setIdativo('S');
        $parcnegocio->setNrorg($this->environment->getNrOrgTrab());
        $parcnegocio->setNmprincipalparc(empty($this->mailInfo['FROMNAME']) ? $this->mailInfo['FROMMAIL'] : $this->mailInfo['FROMNAME']);
        $parcnegocio->setNmsecundariparc(empty($this->mailInfo['FROMNAME']) ? $this->mailInfo['FROMMAIL'] : $this->mailInfo['FROMNAME']);
        $parcnegocio->setNrorginclusao($this->environment->getNrOrgTrab());
        $parcnegocio->setCdoperinclusao($this->environment->getCdOperador());
        $parcnegocio->setCdtipoparcprincipal('INTERLOCUTOR');
        $parcnegocio->setCdtipoinscricao('CPF');
        $parcnegocio->setIdpessoafisica('S');
        $parcnegocio->setIdparcfundido('N');
        $parcnegocio->setIdinstituicao('N');
        $parcnegocio->setDtinclusao(new \DateTime());
        
        $this->entityManager->persist($parcnegocio);
        $this->entityManager->flush();
        
        $this->returnMessage[] = 'Parcnegocio Created';
        $this->keys['NRPARCNEGOCIO'] = $nrparcnegocio;
        
        $this->createComunicaparc();
        if(empty($this->keys['NRSEQINTERLOCUTOR']))
            $this->createCrmInterlocutor();
            
    }
    
    private function createTicket(){
        //CONTATO
        $nrcontato = NovoCodigo::geraCodigo($this->environment->getNrOrgTrab(), "NRSEQCONTATO", 1, 10, false, false);
        $contato = new \Teknisa\Model\Crm\Entities\Contato();
        
        $date = new \DateTime($this->mailInfo['DATE']);
        $date->format('d/m/Y H:i:s');
        
        $contato->setNrcontato($nrcontato);
        $contato->setNrseqcontato($nrcontato);
        $contato->setCdcaso(ltrim($nrcontato, '0'));
        $contato->setNrparcinterlocutor($this->keys['NRPARCNEGOCIO']);
        $contato->setNrparcnegocio(empty($this->keys['NRPARCNEGOCIOPARENT']) ? NULL : $this->keys['NRPARCNEGOCIOPARENT']);
        $contato->setNrorg($this->environment->getNrOrgTrab());
        $contato->setDthrcontato($date);
        $contato->setDsrefecont(substr($this->mailInfo['SUBJECT'], 0, 60));
        $contato->setDsaberturacaso($this->mailInfo['MESSAGE']);
        $contato->setIdtipocontato('CHAMADO');
        $contato->setIdstatuscont('A');
        $contato->setIdtpcontato('P');
        $contato->setIdgerafatura('SIM');
        $contato->setIdorigemticket('EMAIL');
        $contato->setNrseqsituacao(1);
        $contato->setNrorginclusao($this->environment->getNrOrgTrab());
        $contato->setCdoperinclusao($this->environment->getCdOperador());
        $contato->setDtinclusao(new \DateTime());
        
        $this->entityManager->persist($contato);
        $this->entityManager->flush();
        
        $this->returnMessage[] = 'Ticket Created: '.$nrcontato;
        $this->keys['NRSEQCONTATO'] = $nrcontato;
        
    }
    
    private function createComunicaparc() {
        //COMUNICAPARC
        $nrcomunicapac = NovoCodigo::geraCodigo($this->environment->getNrOrgTrab(), "COMUNICAPARC", 1, 10, false, false);
        $comunicaparc = new \Teknisa\Model\Geral\Entities\Comunicaparc();
        
        $comunicaparc->setNrcomunicaparc($nrcomunicapac);
        $comunicaparc->setNrparcnegocio($this->keys['NRPARCNEGOCIO']);
        $comunicaparc->setCdformacomu('05');
        $comunicaparc->setCdcomunicaparc($this->mailInfo['FROMMAIL']);
        $comunicaparc->setIdativo('S');
        $comunicaparc->setNrorg($this->environment->getNrOrgTrab());
        $comunicaparc->setNrorginclusao($this->environment->getNrOrgTrab());
        $comunicaparc->setCdoperinclusao($this->environment->getCdOperador());
        $comunicaparc->setDtinclusao(new \DateTime());
        
        $this->entityManager->persist($comunicaparc);
        $this->entityManager->flush();
        
        $this->returnMessage[] = 'Comunicaparc Created';
        $this->keys['NRCOMUNICAPARC'] = $nrcomunicapac;
        
    }
    
    private function createCrmInterlocutor() {
        //CRM_INTERLOCUTOR
        $nrseqinterlocutor = NovoCodigo::geraCodigo($this->environment->getNrOrgTrab(), "CRM_INTERLOCUTOR", 1, 10, false, false);
        $crminterlocutor = new \Teknisa\Model\Crm\Entities\CrmInterlocutor();
        
        $crminterlocutor->setNrseqinterlocutor($nrseqinterlocutor);
        $crminterlocutor->setNrparcnegocio(empty($this->keys['NRPARCNEGOCIOPARENT']) ? $this->keys['NRPARCNEGOCIO'] : $this->keys['NRPARCNEGOCIOPARENT']);
        $crminterlocutor->setNminterlocutor(empty($this->mailInfo['FROMNAME']) ? $this->mailInfo['FROMMAIL'] : $this->mailInfo['FROMNAME']);
        $crminterlocutor->setEmailinterloc($this->mailInfo['FROMMAIL']);
        $crminterlocutor->setNrcpfinterloc(empty($this->mailInfo['FROMCPF']) ? '' : $this->mailInfo['FROMCPF']);
        $crminterlocutor->setIdativo('S');
        $crminterlocutor->setIdrecebepresente('N');
        $crminterlocutor->setNrorg($this->environment->getNrOrgTrab());
        $crminterlocutor->setNrorginclusao($this->environment->getNrOrgTrab());
        $crminterlocutor->setCdoperinclusao($this->environment->getCdOperador());
        $crminterlocutor->setDtinclusao(new \DateTime());
        
        $this->entityManager->persist($crminterlocutor);
        $this->entityManager->flush();
        
        $this->returnMessage[] = 'CrmInterloc Created';
        $this->keys['NRSEQINTERLOCUTOR'] = $nrseqinterlocutor;
    }
    
    
    /**
     *  Busca na tabela CRM_PARAMETRIZACAO os dados relacionados a IMAP
     * 
     *  @return array                  Array com os dados de IMAP encontrados.
     * 
     *  @throws \Exception             
     *  
    */
    public function getImapInfo(){
        //Se não estiver setada a org, faz a rotina com todas as orgs que possuem a parametrizacao de IMAP
        if($this->environment->getNrOrgTrab() == null){
            $criteria = new Criteria();
            $criteria->where(Criteria::expr()->neq('txuserimap', null));
            $parametrization = $this->entityManager->getRepository('\Teknisa\Model\Crm\Entities\CrmParametrizacao')->matching($criteria)->toArray();    
        }else{
            $parametrization = $this->entityManager->getRepository('\Teknisa\Model\Crm\Entities\CrmParametrizacao')->findOneBy(
                array(
                    'nrorg' => $this->environment->getNrOrgTrab()
                )
            );
        }
        
        return $parametrization;
        
    }
    
    /**
     *  Verifica as instâncias das 3 entidades para descobrir o interlocutor e sua empresa
     * 
     *  @param  object  $comunicaparc       Instância da comunicaparc.            
     *  @param  object  $operador           Instância de operador.            
     *  @param  object  $interlocutor       Instância da interlocutor.
     * 
     * 
    */
    private function setParcAndFindParent($comunicaparc, $operador, $interlocutor){
        if(is_object($operador)){
            //Se encontrar operador, salva ele como interlocutor.
            $this->keys['NRPARCNEGOCIO'] = $operador->getNrparcnegocio();
            $this->returnMessage[] = 'Interloc found on Oper';
            
            $estrutura = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Estruturam')->findOneBy(
                array(
                    'nrestruturam'  => $operador->getNrestruttrab(),
                    'nrorg'         => $this->environment->getNrOrgTrab()
                )
            ); 
            
            if(is_object($estrutura)){
                //Se encontrar estrutura do operador, salva esta como empresa do interlocutor.
                $this->keys['NRPARCNEGOCIOPARENT'] = $estrutura->getNrparcnegocio();
                $this->returnMessage[] = 'Parc name found through Oper';
                //Se encontrar operador e estrutura, não é mais necessário buscar
                return;
            }
        }else{
            $this->returnMessage[] = 'Missing Oper';    
        }
        if(is_object($comunicaparc)){
            //Se não for encontrado operador, mas foi encontrado comunicaparc
            if(empty($this->keys['NRPARCNEGOCIO'])){
                $this->keys['NRPARCNEGOCIO'] = $comunicaparc->getNrparcnegocio();
                $this->returnMessage[] = 'Interloc found on Comunicaparc';
            }
        }
        if(is_object($interlocutor)){
            //Se encontrou interlocutor, busca o parceiro relacionado a este interlocutor e seta ele como empresa do interlocutor.
            $parcnegocio = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Parcnegocio')->findOneBy(
                array(
                    'nrparcnegocio' => $interlocutor->getNrparcnegocio(),
                    'nrorg'         => $this->environment->getNrOrgTrab()
                )
            ); 
             
            if(is_object($parcnegocio)){
                $this->keys['NRPARCNEGOCIOPARENT'] = $parcnegocio->getNrparcnegocio();
                $this->keys['NRSEQINTERLOCUTOR'] = $interlocutor->getNrseqinterlocutor();
                $this->returnMessage[] = 'Parc name found on CrmInterloc';
            }
        }
    }
    
    /**
     *  Faz o upload dos anexos do email no fileServer 
     * e chama a função de inserí-los na tabela CRM_ANEXOCONTATO.     * 
     * 
    */
    private function uploadAttachment(){
        $attachs = is_array($this->mailInfo['ATTACH']) ? $this->mailInfo['ATTACH'] : array($this->mailInfo['ATTACH']);
        foreach($attachs as $attachment){
            if($attachment->b64data !== null){
                $type = explode('.', $attachment->name)[1];
                switch($type){
                    case "pdf": $type="application/pdf"; break;
                    case "exe": $type="application/octet-stream"; break;
                    case "zip": $type="application/zip"; break;
                    case "doc": $type="application/msword"; break;
                    case "xls": $type="application/vnd.ms-excel"; break;
                    case "ppt": $type="application/vnd.ms-powerpoint"; break;
                    case "gif": $type="image/gif"; break;
                    case "png": $type="image/png"; break;
                    case "jpg": $type="image/jpg"; break;
                    case "mp3": $type="audio/mpeg"; break;
                    case "mp4": $type="video/mpeg-4"; break;
                    case "wma": $type="audio/mime"; break;
                    case "txt": $type="text/plain"; break;
                    case "php": // deixar vazio por segurança
                    case "htm": // deixar vazio por segurança
                    case "html": // deixar vazio por segurança
                }
                //Monta o formato b64
                $b64File = 'data'.$type.';base64,'.$attachment->b64data;
                //Calcula o tamanho do arquivo pelo B64;
                $size = (int) (strlen(rtrim($b64File, '=')) * 3 / 4);
                $row = array(
                    'FILE' => array(
                        array(
                            "name" => $attachment->name,
                            "type" => explode('.', $attachment->name)[1],
                            "b64File" => $b64File,
                            "size" => $size
                        )
                    )    
                );
                $this->zhFileUploader->uploadFilesInRow($row, 'FILE', 'crm_anexocontato');
                
                $this->saveInCrmAnexoContato($row['FILE'][0]);
            }
        }
        $this->returnMessage[] = 'Files Uploaded';
    }
    
    private function saveInCrmAnexoContato($row){
        //CRM_ANEXOCONTATO
        $nranexocontato = NovoCodigo::geraCodigo($this->environment->getNrOrgTrab(), "CRM_ANEXOCONTATO", 1, 10, false, false);
        $anexocontato = new \Teknisa\Model\Crm\Entities\CrmAnexocontato();
        
        $anexocontato->setNranexocontato($nranexocontato);
        $anexocontato->setNrseqcontato($this->keys['NRSEQCONTATO']);
        $anexocontato->setNmarquivo($row['name']);
        $anexocontato->setDscaminho($row['path']);
        $anexocontato->setDstipo($row['type']);
        $anexocontato->setVrtamanho($row['size']);
        $anexocontato->setIdorigem('CONTATO');
        $anexocontato->setNrorigem($this->keys['NRSEQCONTATO']);
        $anexocontato->setIdativo('S');
        $anexocontato->setNrorg($this->environment->getNrOrgTrab());
        $anexocontato->setNrorginclusao($this->environment->getNrOrgTrab());
        $anexocontato->setCdoperinclusao($this->environment->getCdOperador());
        $anexocontato->setDtinclusao(new \DateTime());
        
        $this->entityManager->persist($anexocontato);
        $this->entityManager->flush();
        
        $this->keys['NRANEXOCONTATO'] = $nranexocontato;  
    }
    
    public function createOper(&$operador){
        //OPERADOR
        $operador['CDOPERADOR'] = $this->operController->getMaxOperCode(array("NRORG" => $operador['NRORG']));
        $operEntity = new \Teknisa\Model\Geral\Entities\Operador();
        
        $operEntity->setNrparcnegocio($operador['NRPARCNEGOCIO']);
        $operEntity->setCdoperador($operador['CDOPERADOR']);
        $operEntity->setNmoperador($operador['NMOPERADOR']);
        $operEntity->setDsemailoper($operador['DSEMAILOPER']);
        $operEntity->setCdsenhaoperweb(md5($operador['NOVO_NOVA_SENHA']));
        $operEntity->setIdativo('S');
        $operEntity->setIdoperativo('A');
        $operEntity->setIdligextoper('N');
        $operEntity->setIdcadpraprovore('N');
        $operEntity->setIdcadprproceore('N');
        $operEntity->setIdchefereceitam('N');
        $operEntity->setIdtpoperador('C');
        $operEntity->setNrorg($operador['NRORG']);
        $operEntity->setNrorgtrab($operador['NRORG']);
        $operEntity->setNrorginclusao($operador['NRORG']);
        $operEntity->setCdoperinclusao($operador['CDOPERADOR']);
        $operEntity->setDtinclusao(new \DateTime());
        
        $parcnegocio = $this->entityManager->getRepository('\Teknisa\Model\Geral\Entities\Parcnegocio')->findOneBy(
                array(
                    'nrorg' => $operador['NRORG'],
                    'nrparcnegocio'=> $operador['NRPARCNEGOCIO']
                )
            );
        if(is_object($parcnegocio)){
            $parcnegocio->setNrinscricaoparc($operador['NRINSCRICAOPARC']);
            $this->entityManager->persist($parcnegocio);
        }
        
        $this->entityManager->persist($operEntity);
        $this->entityManager->flush();
        
        return $operador;
    }
    
    public function insertInPerf($operador){
        //OPERADOR
        $perfEntity = new \Teknisa\Model\Geral\Entities\Perfacessparc();
        $nrperfacessparc = $this->getMaxNrPerfAcessParc();
        
        $perfEntity->setNrparcnegocio($operador['NRPARCNEGOCIO']);
        $perfEntity->setNrperfacessparc($nrperfacessparc);
        //Perfil de Acesso Padrão - Script 119655
        $perfEntity->setNrperfilacesso($this->getPerfilPadrao($operador['NRORG']));
        $perfEntity->setIdativo('S');
        $perfEntity->setNrorg($operador['NRORG']);
        $perfEntity->setNrorginclusao($operador['NRORG']);
        $perfEntity->setCdoperinclusao($operador['CDOPERADOR']);
        $perfEntity->setDtinclusao(new \DateTime());
        
        $this->entityManager->persist($perfEntity);
        $this->entityManager->flush();
        
        return $operador;
    }
    
    private function getMaxNrPerfAcessParc(){
      $select  = "SELECT MAX(NRPERFACESSPARC) as MAXNRPERFACESSPARC FROM PERFACESSPARC";
                       
        try{
            $std = $this->entityManager->getConnection()->prepare($select);
            $std->execute();
            $maxCode = $std->fetchAll(); 
            
            if(!empty($maxCode))
              return (int)$maxCode[0]['MAXNRPERFACESSPARC'] + 1;
            else
              return 1;
        }
        catch(\Exception $e){
            throw $e;
        };
    }
    
    public function setMailInfo($arrayInfo){
        $this->mailInfo = $arrayInfo;
    }
    
    public function getMailInfo(){
        return $this->mailInfo;
    }
    
    public function setKey($arrayOfKeys){
        $this->keys = array_replace($this->keys, $arrayOfKeys);
    }
    
    public function getKeys(){
        return $this->keys;
    }
    
    public function getReturnMessage(){
        return $this->returnMessage;
    }
    
    public function getPerfilPadrao($nrorg){
        $parametrization = $this->entityManager->getRepository('\Teknisa\Model\Crm\Entities\CrmParametrizacao')->findOneBy(
            array(
                'nrorg' => $nrorg
            )
        );
        
        if(is_object($parametrization)){     
            return $parametrization->getNrperfilacessowebmail();
        }
    }
}