function ChamadoController(templateManager, ScreenService, Utilities, restEngine, General, CRM_Utilities, localStorageEngine, ZHPromise, Messages){
    
    /**
     * Inicializa campos da row.
     * 
     * @param {object} widget widget de chamado
    */
    this.widgetNewRow = function(widget){
        General.widgetNewRow(widget);
        
        widget.currentRow.IDTPCONTATO  = 'P';
        widget.currentRow.IDSTATUSCONT = 'A';
        widget.currentRow.NRPARCNEGOCIORELAC = '';
        widget.currentRow.IDGERAFATURA = 'NAO';
        widget.currentRow.NRLOTEESTQ = '';
        widget.currentRow.CDMOTICONT = '';
        widget.currentRow.NRSEQSITUACAO = '';
        widget.currentRow.NRCLASSIFICACAOCASO = '';
        widget.currentRow.NRPARCINTERLOCUTOR = '';
        widget.currentRow.NMOPERCONT = templateManager.LOGGEDOPERATOR.NMOPERADOR;
        widget.currentRow.CDOPERCONT = templateManager.LOGGEDOPERATOR.CDOPERADOR;
        
        this.setReadOnlyCDLOTEandMOTICONT(widget);
    };
    
    /**
     * Abre a tab de conclusão de chamado, e 
     * coloca ela no modo correto de edição
     * 
     * @param {object} widget widget de chamado
    */
    this.concluirChamado = function(widget){
    
        var conclusaoWidget = templateManager.container.getWidget('WDGT_CONCLUSAO_CHAMADO');
        conclusaoWidget.isVisible = true;
        this.editConclusao(conclusaoWidget);
        widget.openTab('WDGT_CONCLUSAO_CHAMADO');
        conclusaoWidget.newRow();
        //Seta parâmetro para determinar ações no método de cancelar form
        conclusaoWidget.currentRow.CONCLUSAOCHAMADO = true;
        ScreenService.closePopup(true);
    };
    
    /**
     * Abre o swipe de Evolução, procedimento executado
     * pré conclusão do chamado
     * 
     * @param {object} widget widget de chamado
    */
    this.evolucaoPreConclusao = function(widget){
        widget.openTab('WDGT_EVOLUCAO');
        ScreenService.closePopup();
        var evolucaoWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        
        setTimeout(function(){    
            General.widgetNewRow(evolucaoWidget); 
            evolucaoWidget.currentRow.EVOLUCAOPRECONCLUSAO = true;
        }.bind(this), 1000);
        
    };
    
    /**
     * Muda o readonly doS campos MOTICONT e CDLOTE baseado
     * no campo 'Produto'.
     * 
     * @param {object} widget widget de chamado
    */
    this.setReadOnlyCDLOTEandMOTICONT = function(widget, keepValue){
      var currentRow = widget.currentRow;
      var fieldLote = widget.getField('NRLOTEESTQ');
      var fieldMoti = widget.getField('NMMOTICONT');
      var fieldProd = widget.getField('NMPRODUTO');
      
      if(widget.name === "filterPopup"){
        fieldMoti = widget.getField('CDMOTICONT');
      }
      
      if(!Util.isEmptyOrBlank(fieldProd.value()) || !Util.isEmptyOrBlank(currentRow.NRTPARVPROD)){
            if(fieldLote){
                fieldLote.readOnly = false;   
                if(!keepValue)            
                    fieldLote.clearValue();
            }
            if(fieldMoti){
                fieldMoti.readOnly = false;   
                if(!keepValue)            
                    fieldMoti.clearValue();
            }
      }else{
            if(fieldLote){
                fieldLote.readOnly = true;    
                fieldLote.clearValue();
            }
            if(fieldMoti){
                fieldMoti.readOnly = true;   
                fieldMoti.clearValue();
            }
      }
      
      this.setFilterCDLOTEandMOTICONT(widget);
    };
    
    this.setFilterCDLOTEandMOTICONT = function(widget){
        var currentRow = widget.currentRow;  
        var fieldLote = widget.getField('NRLOTEESTQ');
        
        if(fieldLote){
            var filterLote = fieldLote.dataSourceFilter;
            //filtro pelo produto
            if(!Util.isEmptyOrBlank(filterLote[3].value)){
                if(widget.name === "filterPopup")
                    filterLote[3].value = currentRow.CDPRODUTO;
            }
        
        }
        
        var fieldMoti = widget.getField('NMMOTICONT');
        if(widget.name === "filterPopup")
            fieldMoti = widget.getField('CDMOTICONT')
        
        if(fieldMoti){
            var filterMoti = fieldMoti.dataSourceFilter;
            filterMoti[1].value = currentRow.CDPRODUTO;
        }
        
        
        
    };
    
    /**
     * Procedimento para garantir o comportamento correto
     * da widget de conclusao(form) em modo de edição.
     * 
     * @param {object} widget widget de conclusão
    */
    this.editConclusao = function(widget){
        General.widgetEdit(widget);
        if(!widget.backupRow) widget.backupRow = Util.clone(widget.currentRow);
        widget.template = "widget/form.html";
        widget.activate();
    };
    
    /**
     * Procedimento para salvar a conclusão e, 
     * consequentemente a evolução que é feita antes disso.
     * 
     * @param {object} widget widget de conclusão
    */
    this.saveConclusao = function(widget){
        General.widgetSave(widget).
            then(function(response){
                //se tudo der certo no save
                if(!Util.isUndefined(response.dataset.DADOS.NRSEQCONCLUSAO)){
                    //Muda pra modo "view/list"
                    widget.parent.parent.currentRow.DTHRCONCCONT  = CRM_Utilities.getDataHoraAtual();
                    widget.mode = "list";
                    widget.activate();
                    //salvando a evolução 
                    if(!widget.currentRow.KANBAN === true){
                        General.widgetSave(templateManager.container.WIDGETEVOLUCAO);
                        delete templateManager.container.WIDGETEVOLUCAO;
                    }
                    ScreenService.showMessage('Chamado Concluido!');
                    widget.parent.parent.reload();
                    var row = {};
                    
                    row.NRPARCINTERLOCUTOR = widget.parent.parent.currentRow.NRPARCINTERLOCUTOR;
                    row.INTERLOCUTOR       = widget.parent.parent.currentRow.NMPARCINTERLOCUTOR;
                    row.NRSEQCONTATO       = widget.parent.parent.currentRow.NRSEQCONTATO;
                    row.PRODUTO            = widget.parent.parent.currentRow.NMPRODUTO;
                    row.CLIENTE            = widget.parent.parent.currentRow.NMPARCNEGOCIO;
                    row.REFERENCIA         = widget.parent.parent.currentRow.DSREFECONT;
                    row.DATACONCLUSAO      = widget.parent.parent.currentRow.DTHRCONCCONT;
                    row.EMAIL              = widget.parent.parent.currentRow.EMAIL;
                    row.NRORG              = widget.currentRow.NRORG;
                    row.RESPONSAVEL        = widget.currentRow.NMOPERCONCLUSAO;
                    row.DESCCONCLUSAO      = widget.currentRow.DESCCONCLUSAO;
                    
                    
                    var paramsRestEngine = {
                        "requestType": "Row",
                        "row": row
                    };
                    restEngine.post('/notificaInterlocutorChamado', paramsRestEngine).then(function(){}).
                        catch(function(fail){
                            ScreenService.showMessage('Falha ao Notificar o Solicitante.');
                        }.bind(this));
                }else{
                    //se algo der errado
                    General.widgetCancelForm(templateManager.container.WIDGETEVOLUCAO);
                    delete templateManager.container.WIDGETEVOLUCAO;    
                }
            }.bind(this));
    };
    
    /**
     * Procedimento para cancelar a edição da conclusão 
     * garantindo o comportamento correto da widget conclusão(form).
     * 
     * @param {object} widget widget de conclusão
    */
    this.cancelConclusao = function(widget){
        //Se não for o primeiro save de conclusão, cancela e bota em modo "view/list"
        if(!widget.currentRow.CONCLUSAOCHAMADO){
            General.widgetCancelForm(widget);
            widget.mode = "list";
            widget.activate();
        }else{
            //Se for a primeira ediçãod a conclusão, cancela e volta para a tab principal
            var widgets = widget.parent.parent.widgets;
    		General.changeWidgetAcessibility(widgets.concat(widget.parent.parent), true);
    		widget.mode = "list";
    		widget.template = "widget/form-view.html";
    		widget.editing = false;
    		widget.activate();
    		widget.parent.parent.openTab('WDGT_CHAMADO');
    		delete widget.currentRow.CONCLUSAOCHAMADO;
        }
    };
    
    /**
     * Seta os dados vindos da requisição na currentRow
     * do form.
     * 
     * @param {object} widget widget de conclusão
    */
    this.setConcInCurrentRow = function(widget){
        if(widget.dataSource.data.length > 0 ){
            widget.currentRow = widget.dataSource.data[0];
        } 
    };
    
    /**
     * Procedimento para garantir o fucnionamento correto,
     * do field colorido no grid.
     * 
     * @param {object} widget widget de chamado
    */
    this.classificacaoOnChange = function(widget){
        var NRCLASSIFICACAOCASO = widget.currentRow.NRCLASSIFICACAOCASO;
        var fieldColor     = widget.getField("DSCLASSIFICACAOCASO");
        
        //Tira a cor do field color se não houver termômetro selecionado
        if(Util.isEmptyOrBlank(NRCLASSIFICACAOCASO)){
            fieldColor.clearValue();
            $('.inner-color').removeAttr('style');
            $('.color-field-view .zh-select-multiple').addClass('zh-field-value');
        }
    };
    
   /**
     * metodo chamado para cada td do GRID, percorre o HTML e colore
     * o field de termômetro.
     * 
     * @param {object} row row do grid a ser percorrida.
     *        {array} elemento jquery representando cada td da row.
    */
    this.setColorOfRowBasedOnPrioridade = function(row, element){
        var color   = row.DSCORPRIORIDADE;
        var fieldsHTML = $(element[0]);
        if(fieldsHTML.attr('data-zh-field-name') === "DSPRIORIDADECLI"){
            //style padrão de fields zeedhi
            fieldsHTML.css({color:color});
        }
    };
    
    /**
     * Abre o popup de transferência e inicia a widget de evolução.
     * 
     * @param {object} widget widget de chamado
    */
    this.transferirChamado = function(widget){
        
        
        var evolWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        var transferWidget = templateManager.container.getWidget('WDGT_TRANSFERIR_CHAMADO');
        
        evolWidget.newRow();
        transferWidget.newRow();
        
        if(widget.name === "WDGT_CHAMADO_KANBAN")
            transferWidget.kanban = true;
        
        var parcFilter = transferWidget.getField('NMINTERLOCUTOR').dataSourceFilter;
        var parcFilterValue;
        if(Array.isArray(widget.currentRow)){
            parcFilterValue = [];
            widget.currentRow.forEach(function(ticket){
                parcFilterValue.push(ticket.NRPARCNEGOCIO);
            });
            transferWidget.getField('NMINTERLOCUTOR').dataSourceFilter[CRM_Utilities.getFilterPosition(parcFilter, 'NRPARCNEGOCIO')].value =  parcFilterValue;
            transferWidget.getField('NMINTERLOCUTOR').dataSourceFilter[CRM_Utilities.getFilterPosition(parcFilter, 'NRPARCNEGOCIO')].operator = "IN";
        }else{
            parcFilterValue = widget.currentRow.NRPARCNEGOCIO;
            transferWidget.getField('NMINTERLOCUTOR').dataSourceFilter[CRM_Utilities.getFilterPosition(parcFilter, 'NRPARCNEGOCIO')].value =  parcFilterValue;
            transferWidget.getField('NMINTERLOCUTOR').dataSourceFilter[CRM_Utilities.getFilterPosition(parcFilter, 'NRPARCNEGOCIO')].operator = "=";
        }
        
        ScreenService.openPopup(transferWidget);
        
    };
    
    /**
     * Inicializa os campos necessários para a criação da evolução
     * de transferência, evolui e inicia o procedimento de transferência de
     * operador
     * 
     * @param {object} widget widget de chamado
    */
    this.completeTransferecia = function(widget, kanban){
        
        var ticketWdgt = templateManager.container.getWidget('WDGT_CHAMADO');
        var chamadoCurrentRows = ticketWdgt.currentRow; 
        if(Util.isEmptyOrBlank(Object.keys(chamadoCurrentRows)) || kanban){
            ticketWdgt = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
            chamadoCurrentRows = ticketWdgt.currentRow; 
        }
        var evolWidget = templateManager.container.getWidget('WDGT_EVOLUCAO'); 
        
        if(!Array.isArray(chamadoCurrentRows)){
            chamadoCurrentRows = [chamadoCurrentRows];
        }
        
        if(!Array.isArray(evolWidget.currentRow)){
            evolWidget.currentRow = [evolWidget.currentRow];
            chamadoCurrentRows.forEach(function(chamadoCurrentRow, index){
                evolWidget.currentRow[index] = angular.copy(evolWidget.currentRow[0]);
            });
        }
    
        chamadoCurrentRows.forEach(function(chamadoCurrentRow, index, chamadoCurrentRows){
            evolWidget.currentRow[index].CDOPEREVOL = templateManager.LOGGEDOPERATOR.CDOPERADOR;
            evolWidget.currentRow[index].DTHREVOLCONT = CRM_Utilities.getDataHoraAtual();
            evolWidget.currentRow[index].NRCONTATO = evolWidget.currentRow[index].NRSEQCONTATO = chamadoCurrentRow.NRSEQCONTATO;
            evolWidget.currentRow[index].NRORG = templateManager.LOGGEDOPERATOR.NRORG;
            evolWidget.currentRow[index].IDSENEVOCONT = 'E';
            evolWidget.currentRow[index].NRSEQINTERLOCUTOR = widget.currentRow.NRSEQINTERLOCUTOR;
            
            //required fields in evolWidget
            evolWidget.getField('NMTIPOCONT').setValue(widget.currentRow.NMTIPOCONT);
            evolWidget.getField('NMMOTIEVOCON').setValue(widget.currentRow.NMMOTIEVOCON);
            evolWidget.getField('NMSEQRESULEVOL').setValue(widget.currentRow.DESCRESULEVOL);
            
            evolWidget.currentRow[index].CDTIPOCONT = widget.currentRow.CDTIPOCONT;
            evolWidget.currentRow[index].NRSEQRESULEVOL = widget.currentRow.NRSEQRESULEVOL;
            evolWidget.currentRow[index].CDMOTIEVOCON = widget.currentRow.CDMOTIEVOCON;
            evolWidget.currentRow[index].IDTPEVOLCONT = 'P';
            evolWidget.currentRow[index].NRRELACIONAPARC = '';
            evolWidget.currentRow[index].IDANEXOEVOLCONT = 'N';
            evolWidget.currentRow[index].IDGERAFATURA = 'NAO';
            evolWidget.currentRow[index].NRSEQFILACONTA = '';
            
            if(!Util.isEmptyOrBlank(chamadoCurrentRow.CDOPERCONT))
                evolWidget.currentRow[index].TXEVOLCONT = 'Transferência do chamado: '+ chamadoCurrentRow.CDCASO +' do operador: '+ chamadoCurrentRow.NMOPERCONT +' para o operador: '+ widget.currentRow.NMOPERADOR;
            else
                evolWidget.currentRow[index].TXEVOLCONT = 'Transferência do chamado: '+ chamadoCurrentRow.CDCASO +' para o operador: '+ widget.currentRow.NMOPERADOR;
        });
        
        var paramsRestEngine = {
            "requestType": "DataSet",
            "dataset": evolWidget.currentRow
        };
        
        restEngine.post('/evolcont_contato/save', paramsRestEngine).  
            then(function(response){
                if(!Array.isArray(widget.currentRow)){
                    widget.currentRow = [widget.currentRow];
                    chamadoCurrentRows.forEach(function(chamadoCurrentRow, index){
                        widget.currentRow[index] = angular.copy(widget.currentRow[0]);
                    });
                }
                evolWidget.currentRow.forEach(function(row, index){
                    widget.currentRow[index].NRSEQCONTATO = evolWidget.currentRow[index].NRSEQCONTATO;
                    widget.currentRow[index].NRORG = evolWidget.currentRow[index].NRORG;
                    widget.currentRow[index].CDOPEREVOL = evolWidget.currentRow[index].CDOPEREVOL;
                    widget.currentRow[index].DTHREVOLCONT = evolWidget.currentRow[index].DTHREVOLCONT
                });
                this.transfereOperador(widget.currentRow, ticketWdgt.name);
            }.bind(this));    
    };
    
    /**
     * fecha os popups
     * 
     * @param {object} widget widget de chamado
    */
    this.cancelTransferencia = function(widget){
        if(widget.name === "WDGT_MOVE_TICKET"){
            var ticketWdgt = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
            this.updateColumns(ticketWdgt);
        }
        ScreenService.closePopup(true);    
    };
    
    /**
     * Transfere o chamado
     * 
     * @param {object} widget widget de chamado
    */
    this.transfereOperador = function(currentRow, widgetName){
        
        var paramsRestEngine = {
            "requestType": "DataSet",
            "dataset": currentRow
        };
        
        restEngine.post('/transfereOperador', paramsRestEngine).  
            then(function(response){
                if(response.dataset){
                    ScreenService.showMessage('Ticket Transferido!');
                    ScreenService.closePopup(true);
                    ScreenService.closeSwipe(true); 
                    templateManager.container.getWidget(widgetName).reload();
                    templateManager.container.getWidget(widgetName).dataSource.clearCheckedRows();
                }
            }.bind(this)).
            catch(function(fail){
                ScreenService.showMessage('Falha ao transferir o Chamado.');
            }.bind(this));
    };
    
    /**
     * Decide se as actions de editar e excluir estarão
     * visíveis(verifica se o chamado está fechado).
     * 
     * @param {object} widget widget de chamado
    */
    this.controlActionsVisibilityByConclusao = function(widget){
        if(!Util.isEmptyOrBlank(widget.currentRow.DTHRCONCCONT)){
            this.setActionsVisibilityByConclusao(widget, false);    
        }else{
            this.setActionsVisibilityByConclusao(widget, true);    
        }
    };
    
    /**
     * Seta a visibilidade das actions recursivamente para 
     * todas as widgets exceto a de conclusão
     * 
     * @param {object} widget widget de chamado
    */
    this.setActionsVisibilityByConclusao = function(widget, visibility){
        if(!Util.isEmptyOrBlank(widget.widgets)){
            widget.widgets.forEach(function(inWidget){
                this.setActionsVisibilityByConclusao(inWidget, visibility);
            }.bind(this));
        }
        if(!widget.preventHide){
            if(widget.actions){
                widget.actions.forEach(function(action){
                    if(!action.preventHide)
                        action.isVisible = visibility;
                });
            }
        }
    };
    
    /**
     * Garantindo o reload da widget após os ave
     * 
     * @param {object} widget widget de chamado
    */
    this.widgetSave = function(widget){
        if(Util.isEmptyOrBlank(widget.currentRow.CDOPERCONT))
            widget.currentRow.CDOPERCONT = templateManager.LOGGEDOPERATOR.CDOPERADOR;
        if(widget.currentRow.IDPERMITECONCLUSAO === 'S'){
            widget.currentRow.DTHRCONCCONT = CRM_Utilities.getDataHoraAtual();
        }
        if(widget.name === "WDGT_CHAMADO_KANBAN"){
            delete widget.currentRow.resources;
            delete widget.currentRow.FROMCOLUMN;
            delete widget.currentRow.TOCOLUMN;
            General.widgetSave(widget).then(function(response){
                widget.childTemplate = "widget/form-view.html";
                widget.reload().then(function(response){
                    this.defaultFilters(widget);
                    this.controlActionsVisibilityByConclusao(widget);
                    this.setSituacaoOnKanbanColumns(widget);
                    this.setPeopleOnCurrentRow(widget);
                }.bind(this));
            }.bind(this));
            
        }else{
            CRM_Utilities.setColumnToReturnInCurrentRow(widget.currentRow, "DTULTATU");
            General.widgetSave(widget).then(function(response){
                widget.reload().then(function(){
                    widget.currentRow.DTULTATU = response.dataset.columnsToReturn.DTULTATU;
                    this.setPeopleOnCurrentRow(widget);
                }.bind(this));
            }.bind(this));    
        }
    };
    
    
    
    /**
     * Seta os filtros padrões
     * 
     * @param {object} widget widget de chamado
    */
    this.defaultFilters = function(widget){
        var filter = widget.__filterPopup;
        if(filter){
            filter.currentRow.CDOPERCONT = [templateManager.LOGGEDOPERATOR.CDOPERADOR];
            filter.currentRow.NMOPERCONT = [templateManager.LOGGEDOPERATOR.NMOPERADOR];
            filter.getField('CDOPERCONT').show();
        }
    };
    
    this.setVisibilityFieldsComunica = function(widget){
        if(!widget.currentRow.__is_new){
            widget.getField('DSCAMINHO').isVisible = false;
            //widget.getField('TELEFONE').isVisible = false;
            //widget.getField('EMAIL').isVisible = false;
        }else if(widget.currentRow.__is_new){
            widget.getField('DSCAMINHO').isVisible = true;
            //widget.getField('TELEFONE').isVisible = true;
            //widget.getField('EMAIL').isVisible = true;
        }
    };
    
    
    /**
     * Função para abrir a widget de seleção
     * de arquivos.
    */
    this.openAttachSwipe = function(){
        
        var fileUploadWidget = templateManager.container.getWidget('WDGT_SELECAO_ARQUIVOS_CHAMADO');
        
        fileUploadWidget.isVisible = true;
        ScreenService.openSwipe(fileUploadWidget);
            
    };
    
    this.nullFilter  = function(widget, nullFieldName, fieldName, widName){
        var fieldNull = widget.getField(nullFieldName);
        var field = widget.getField(fieldName);
        
        if(fieldNull.value() === "IS_NULL"){
            field.clearValue();
            field.readOnly = true;
        }else{
            field.clearValue();
            field.readOnly = false;
            if(fieldName === 'CDOPERCONT')
                this.defaultFilters(templateManager.container.getWidget(widName));
        }
    };
    
    this.buildFilter = function(widget){
        var dataSourceFilter = widget.dataSourceFilter;
        var indexResp, newFilter;
        var filters;
        
        dataSourceFilter.forEach(function(filter, index, dataSourceFilter){
            if(filter.name === "RESPNULL"){
                newFilter = {
                    "name":"CDOPERCONT",
                    "static":true,
                    "operator":filter.value
                };
                if(filter.value === "IS_NULL")
                    filters = dataSourceFilter.filter(function(f){return f.name !== "CDOPERCONT" && f.name !== "RESPNULL"});
                else
                    filters = dataSourceFilter.filter(function(f){return f.name !== "RESPNULL"});    
                    
                filters.push(newFilter);
            }
            if(filter.name === "IDNOMETAG"){
                if(Array.isArray(filter.value)){
                    filter.value = filter.value.join(',');
                    filter.value = ','+filter.value+',';
                }
            }
            if(filter.name === "NRPARCNEGOCIORELAC"){
                if(Array.isArray(filter.value)){
                    filter.value = filter.value.join(',');
                    filter.value = ','+filter.value+',';
                }
            }
        });
        if(filters)
            widget.dataSourceFilter = filters;
        this.filterClieNull(widget);
    };
    
    this.filterClieNull = function(widget){
        var dataSourceFilter = widget.dataSourceFilter;
        var newFilter;
        var filters;
        
        dataSourceFilter.forEach(function(filter, index, dataSourceFilter){
            if(filter.name === "CLIENULL"){
                newFilter = {
                    "name":"NRPARCNEGOCIO",
                    "static":true,
                    "operator":filter.value
                };
                if(filter.value === "IS_NULL")
                    filters = dataSourceFilter.filter(function(f){return f.name !== "NRPARCNEGOCIO" && f.name !== "CLIENULL"});
                else
                    filters = dataSourceFilter.filter(function(f){return f.name !== "CLIENULL"});    
                    
                filters.push(newFilter);
            }
        });
        if(filters)
            widget.dataSourceFilter = filters;
    };
            
    
    /**
     * Muda o readonly do campo Sistema baseado
     * no campo 'Classificação'.
     * 
     * @param {object} widget widget de chamado
     * Desativado 17/01/18 - Mudanças na rotina de produto
    */
    /**
    this.setReadOnlySistema = function(widget, keepValue){
        var currentRow = widget.currentRow;
        var fieldSistema = widget.getField('NMSISTEMA');
        
        if(widget.name === "filterPopup")
        fieldSistema = widget.getField('CDSISTEMA')
        
        if(!Util.isEmptyOrBlank(currentRow.NRCLASSIFICACAOCASO)){
            if(fieldSistema){
                fieldSistema.readOnly = false;    
                if(!keepValue)
                    fieldSistema.clearValue();
            }
        }else{
            if(fieldSistema){
                fieldSistema.readOnly = true;    
                fieldSistema.clearValue();
            }    
        }
      
      this.setFilterSistema(widget);
    };
    this.setFilterSistema = function(widget){
        var currentRow = widget.currentRow;  
        var fieldSistema = widget.getField('NMSISTEMA');

        if(widget.name === "filterPopup")
            fieldSistema = widget.getField('CDSISTEMA')
        
        if(fieldSistema){
            var filterSistema = fieldSistema.dataSourceFilter;
            filterSistema[2].value = currentRow.NRCLASSIFICACAOCASO;
        }
        
        this.setReadOnlyCDLOTEandMOTICONT(widget, true);
        
    };
    */
    
    /**
     *  Desabilita o clique em níveis menores que o máximo
     *  do select tree/grid e muda a label do field,
     *  adicionando a hierarquia da árvore selecionada.
    */
    this.getBreadcrumbForProdField = function(args, widgetName){
        
        var parentWidget       = templateManager.container.getWidget(widgetName);
        var parentWidgetFilter = parentWidget.__filterPopup ? parentWidget.__filterPopup : false;
            
        var currentRow   = parentWidget.currentRow;
        if(parentWidgetFilter)
            var currentRowFilter   = parentWidgetFilter.currentRow;
        
        if(args.row.TIPO !== "P"){
            ScreenService.openPopup(args.owner);
        }else{
            
            this.setProdInCurrentRow(currentRow, args);
            
            if(parentWidgetFilter)
                this.setProdInCurrentRow(currentRowFilter, args);
            
            if(widgetName === 'WDGT_CHAMADO' || widgetName === 'WDGT_ABERTURA_CHAMADO' || widgetName === 'WDGT_CHAMADO_KANBAN'){
                this.setFilterCDLOTEandMOTICONT(parentWidget);
                
                var fieldMoti = parentWidget.getField('NMMOTICONT');
                if(fieldMoti)
                    fieldMoti.readOnly = false;
                    
                var fieldLote = parentWidget.getField('NRLOTEESTQ');     
                if(fieldLote)    
                    parentWidget.getField('NRLOTEESTQ').readOnly = false;
                
                if(parentWidgetFilter){
                    var fieldMoti = parentWidgetFilter.getField('CDMOTICONT');
                    if(fieldMoti)
                        fieldMoti.readOnly = false;
                        
                    var fieldLote = parentWidgetFilter.getField('NRLOTEESTQ');     
                    if(fieldLote)    
                        fieldLote.readOnly = false;
                    
                }
            }
            
        }
    };
    
    //Seta os dados do produto na currentRow, com breadcrumb
    this.setProdInCurrentRow = function(currentRow, args){
        currentRow.NMPRODUTO    = args.row._breadcrumb.replace(new RegExp(' > ', 'g'), ' -- ') + ' -- ' + args.row.NAME;
        currentRow.CDPRODUTO    = args.row.ID;
        currentRow.NRTPARVPROD  = args.row.NRTPARVPROD;
        currentRow.NRARVOREPROD = this.findNrArvoreProd(args.dataSource, args.dataSource.indexOf(args.row));
    };
    
    /**
     * Métodos para encontrar a estrutura da árvore
     * em que o produto escolhido se encontra
     * 
     * 17/01/18 - Arthur Melo
    *
    *   Encontra e retorna o index do produto passado no array de produtos e níveis de árvore
    */
    this.findParentOnArray = function(rows, element){
        var index = rows.find(function(e, i, a){
            if(e.ID === element.PARENT && e.NRTPARVPROD === element.NRTPARVPROD) 
                return e;
        }.bind(this));
        return rows.indexOf(index);
    }
    /*
    *   Encontra o NRARVOREPROD do menor nível na estrutura da árvore do produto
    */
    this.findNrArvoreProd = function(rows, chosenIndex){
        if(parseInt(rows[chosenIndex].PARENT) === parseInt(rows[chosenIndex].NRTPARVPROD))    
            return rows[chosenIndex].ID.replace(rows[chosenIndex].NRTPARVPROD.toString(), '');
        else
            return this.findNrArvoreProd(rows, this.findParentOnArray(rows, rows[chosenIndex]));
    };
    
    /**
     * Métodos para encontrar abrir a árvore
     * do produto que está selecionado
     * 
     * 18/01/18 - Arthur Melo
    *
    *   Encontra o produto na árvore e abre sua árvore
    */
    this.findProdcutAndOpenTree = function(args, row){
        var rows = args.dataSource;
        
        if(!Util.isEmptyOrBlank(row.CDPRODUTO) && !Util.isEmptyOrBlank(rows)){
            
            var parent = row.NRTPARVPROD;
            var id = row.CDPRODUTO;
            
            var product = rows.find(function(e, i, a){
                if(e.ID == id && e.NRTPARVPROD == parent)
                    return e;
            });
            
            this.openTree(rows, product, id);
            
            templateManager.container.getActiveWidget().dataSource.data = rows;
        }
        /*Gambs pra conseguir filtrar o tree grid*/
        args.owner.dataSource.data = Util.isEmptyOrBlank(args.owner.plainTreeData) ? args.owner.dataSource.data : args.owner.plainTreeData;
    };
    /*
    *   Abre todos os parents d produto
    */
    this.openTree = function(rows, product, cdproduto){
        if(product.ID != cdproduto)
            product._opened = true;
            
        if(product.PARENT !== null){
            this.openTree(rows, rows[this.findParentOnArray(rows, product)]);
        }
        
        return;
    };
    
    /*
    *   Cria o filtro de tipo de árvore, quando
    *   o produto é selecionado
    */
    this.filterByTree = function(widget){
        var dataSourceFilter = widget.dataSourceFilter;
        var newFilter, productFiltered = false;
        
        dataSourceFilter.forEach(function(filter, index, dataSourceFilter){
            if(filter.name === "CDPRODUTO"){
                newFilter = {
                    "name":"NRTPARVPROD",
                    "static":true,
                    "operator":'=',
                    "value":widget.__filterPopup.currentRow.NRTPARVPROD
                };
                
                dataSourceFilter.push(newFilter);
                
                productFiltered = true;
            } 
        });
        
        if(!productFiltered){
            dataSourceFilter.find(function(e, i, a){
                if(e.name === "NRTPARVPROD")
                    a.splice(i, 1);
            });   
        }
    };
    /**
     * Fim
    */
    
    this.testImap = function(widget){
        restEngine.post('product#/insertTicketsInMail', {});
    };
    
    this.changeAnalistaReadOnly = function(widget){
        var analistaField = widget.getField('NMOPERCONT');
        if(Util.isEmptyOrBlank(widget.currentRow.CDOPERCONT)){
            analistaField.readOnly = false;
        }else{
            analistaField.readOnly = true;
        }
    }
    
    /**
     * KANBAN 
    */
    
    this.setSituacaoOnKanbanColumns = function(widget, reload){
        var paramsRestEngine = {
            "requestType": "FilterData",
            filter:[
                {
                    name:"NRORG",
                    operator:"IN",
                    value:[getLocalVar('CRM_NRORG'), getLocalVar('CRM_NRORGPADRAO')]
                }
            ]
        };
        if(Util.isEmptyOrBlank(widget.columns)){
            restEngine.post('product#/crud/crm_situacao', paramsRestEngine).
                then(function(response){
                    widget.columns = [];
                    response.dataset.crm_situacao.forEach(function(situacao){
                        if(situacao.IDEXIBEKANBAN === 'S' && situacao.IDATIVO === 'S'){
                            widget.columns.push({
                                id:situacao.NRSEQSITUACAO,
                                order:situacao.NRORDEM,
                                idpermiteconclusao:situacao.IDPERMITECONCLUSAO,
                                movable:situacao.IDPERMITECONCLUSAO === 'N' ? true : false, 
                                label:situacao.DSSITUACAOCLI
                            });
                        }
                    });
                    this.updateColumns(widget);
                }.bind(this));
        }else{
            this.updateColumns(widget);
        }
    };
    
    this.updateColumns = function(widget){
        widget.dataSource.data.forEach(function(ticket){
            ticket.column = ticket.NRSEQSITUACAO;   
            ticket.resources = [{
                photo: ticket.OPERCONTFOTO,
                name:  ticket.NMOPERCONT ? ticket.NMOPERCONT : 'Sem Analista.'
            }];
            var column = widget.columns.find(function(column) {if(column.id === ticket.column) return column});
            ticket.movable =  column.movable;
        });
        this.rebuildKanban(widget);
    };
    
    this.setColorRibbon = function(row, element){
        var color    = row['DSCORPRIORIDADE'];
        var cssClass = row['DSPRIORIDADE']; 
        element.addClass(cssClass);
        $('.'+cssClass+' .kanban-ticket-ribbon').css({
          'background-color': color
        })
        $('.'+cssClass+' .kanban-ticket-ribbon .before').css({
          'border-color': color+' transparent transparent'
        })
        $('.'+cssClass+' .kanban-ticket-ribbon .after').css({
          'border-color': 'transparent ' +color+ ' transparent transparent'
        })
    };
    
    this.fixContainerSize = function(){
        var widget = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
        //Conserta tamanho do container de acordo com a quantidade de colunas (O tamanho da task do kanban foi alterado)
        $('.zh-kanban-column-container').width(332*widget.columns.length);
    };
    
    this.findColumnById = function(columns, id){
        return columns.find(function(column, index, columns){
            if(parseInt(column.id) === parseInt(id))
                return column;
        });
    };
    
    /**
     * Abre o popup de mudança de status e inicia a widget de evolução.
     * 
     * @param {object} widget widget de chamado
    */
    this.ticketStatusChange = function(widget, e){
        
        var evolWidget = templateManager.container.getWidget('WDGT_EVOLUCAO');
        var moveWidget = templateManager.container.getWidget('WDGT_MOVE_TICKET');
        
        evolWidget.newRow();
        moveWidget.newRow();
        
        moveWidget.currentRow = e.item;
        moveWidget.currentRow.FROMCOLUMN = this.findColumnById(widget.columns, e.columns.from);
        moveWidget.currentRow.TOCOLUMN = this.findColumnById(widget.columns, e.columns.to);
        moveWidget.label = 'Mudança de Status - De: '+moveWidget.currentRow.FROMCOLUMN.label+' Para: '+moveWidget.currentRow.TOCOLUMN.label;
        moveWidget.currentRow.TXEVOLCONT = 'Mudança de Status do Ticket: '+ moveWidget.currentRow.CDCASO +'\nDe: '+ moveWidget.currentRow.FROMCOLUMN.label +' \nPara : '+ moveWidget.currentRow.TOCOLUMN.label;
        var parcFilter = moveWidget.getField('NMINTERLOCUTOR').dataSourceFilter;
        moveWidget.getField('NMINTERLOCUTOR').dataSourceFilter[CRM_Utilities.getFilterPosition(parcFilter, 'NRPARCNEGOCIO')].value =  e.item.NRPARCNEGOCIO;
        ScreenService.openPopup(moveWidget);
        
    };
    
    this.ticketBeforeMove = function(widget, args){
        if(args.e.columns.to !== "1" && args.e.columns.from !== args.e.columns.to){
            widget.currentRow = args.e.item;
            if(!widget.isValid()){
                ScreenService.showMessage(Messages.IMPOSSIVEL_MANUSEAR_EVENTO('Favor Completar o Ticket com as Informações Básicas'));
                return false;
            }
            this.ticketStatusChange(widget, args.e);    
        }else{
            if(args.e.columns.from !== "1" && args.e.columns.from !== args.e.columns.to)
                ScreenService.showMessage(Messages.IMPOSSIVEL_MANUSEAR_EVENTO("Não é possível voltar um ticket para a situação 'Aberto'"));
            return false;
        }
    };
    
    /**
     * Inicializa os campos necessários para a criação da evolução
     * de mudança de status e evolui.
     * 
     * @param {object} widget widget de ticket
    */
    this.completeStatusChange = function(widget){
        
        var ticketCurrentRow = templateManager.container.getWidget('WDGT_MOVE_TICKET').currentRow; 
        var evolWidget = templateManager.container.getWidget('WDGT_EVOLUCAO'); 
        
        evolWidget.currentRow.CDOPEREVOL = templateManager.LOGGEDOPERATOR.CDOPERADOR;
        evolWidget.currentRow.DTHREVOLCONT = CRM_Utilities.getDataHoraAtual();
        evolWidget.currentRow.NRCONTATO = evolWidget.currentRow.NRSEQCONTATO = ticketCurrentRow.NRSEQCONTATO;
        evolWidget.currentRow.NRORG = templateManager.LOGGEDOPERATOR.NRORG;
        evolWidget.currentRow.IDSENEVOCONT = 'E';
        
        //required fields in evolWidget
        evolWidget.getField('NMTIPOCONT').setValue(widget.currentRow.NMTIPOCONT);
        evolWidget.getField('NMMOTIEVOCON').setValue(widget.currentRow.NMMOTIEVOCON);
        evolWidget.getField('NMSEQRESULEVOL').setValue(widget.currentRow.DESCRESULEVOL);
        
        evolWidget.currentRow.CDTIPOCONT = widget.currentRow.CDTIPOCONT;
        evolWidget.currentRow.NRSEQRESULEVOL = widget.currentRow.NRSEQRESULEVOL;
        evolWidget.currentRow.CDMOTIEVOCON = widget.currentRow.CDMOTIEVOCON;
        evolWidget.currentRow.IDTPEVOLCONT = 'P';
        evolWidget.currentRow.NRRELACIONAPARC = '';
        evolWidget.currentRow.IDANEXOEVOLCONT = 'N';
        evolWidget.currentRow.IDGERAFATURA = 'NAO';
        evolWidget.currentRow.NRSEQFILACONTA = '';
        evolWidget.currentRow.TXEVOLCONT = ticketCurrentRow.TXEVOLCONT;
        evolWidget.currentRow.NRSEQINTERLOCUTOR = ticketCurrentRow.NRSEQINTERLOCUTOR;
        
        General.widgetSave(evolWidget).
            then(function(response){
                widget.currentRow.NRSEQCONTATO = evolWidget.currentRow.NRSEQCONTATO;
                widget.currentRow.NRORG = evolWidget.currentRow.NRORG;
                widget.currentRow.CDOPEREVOL = evolWidget.currentRow.CDOPEREVOL;
                widget.currentRow.DTHREVOLCONT = evolWidget.currentRow.DTHREVOLCONT;
                
                var ticketWdgt = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
                ticketWdgt.currentRow = ticketCurrentRow;
                ticketWdgt.currentRow.NRSEQSITUACAO = ticketWdgt.currentRow.TOCOLUMN.id;
                if(Util.isEmptyOrBlank(ticketWdgt.currentRow.CDOPERCONT))
                    ticketWdgt.currentRow.CDOPERCONT = templateManager.LOGGEDOPERATOR.CDOPERADOR;
                if(ticketWdgt.currentRow.TOCOLUMN.idpermiteconclusao === 'S'){
                    ticketWdgt.currentRow.DTHRCONCCONT = CRM_Utilities.getDataHoraAtual();
                    this.beforeConcStatus(evolWidget.currentRow.NRSEQRESULEVOL, evolWidget.currentRow.TXEVOLCONT);
                }
                delete ticketWdgt.currentRow.resources;
                delete ticketWdgt.currentRow.FROMCOLUMN;
                delete ticketWdgt.currentRow.TOCOLUMN;
                General.widgetSave(ticketWdgt).then(function(){
                    ScreenService.closePopup(true);
                    ticketWdgt.mode = "list";
                    ticketWdgt.activate();
                    this.setSituacaoOnKanbanColumns(ticketWdgt);
                }.bind(this));
            }.bind(this)).catch(function(){
                var ticketWdgt = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
                this.setSituacaoOnKanbanColumns(ticketWdgt);
            }.bind(this));    
    };
    
    this.beforeConcStatus = function(result, desc){
        var widConc = templateManager.container.getWidget('WDGT_CONCLUSAO_CHAMADO');
        widConc.currentRow = {
            CDOPERCONCLUSAO: templateManager.LOGGEDOPERATOR.CDOPERADOR,
            NMOPERCONCLUSAO: templateManager.LOGGEDOPERATOR.NMOPERADOR,
            CONCLUSAOCHAMADO: true,
            DTCONCLUSAO: CRM_Utilities.getDataHoraAtual(),
            NRORG: templateManager.LOGGEDOPERATOR.NRORG,
            NRSEQCONTATO: widConc.parent.parent.currentRow.NRSEQCONTATO,
            DESCCONCLUSAO: 'Chamado Concluido!\n'+ desc,
            IDATIVO:'S',
            NRSEQRESULTEVOL:result,
            __is_new: true,
            KANBAN: true
        };
        this.saveConclusao(widConc);
    };
    
    this.setTagsOnCurrentRow = function(widget, parcOrTicket){
        var currentRow = widget.currentRow;
        var code;
        if(parcOrTicket === 'PARCNEGOCIO'){
            code = currentRow.NRPARCNEGOCIO;
        }else if(parcOrTicket === 'CONTATO'){
            code = currentRow.NRSEQCONTATO
        }
        this.getTags(code, parcOrTicket).then(function(response){
            if(!Util.isEmptyOrBlank(response)){
                widget.currentRow.IDNOMETAG = [];
                widget.currentRow.DSNOMETAG = [];
                response.forEach(function(tag){
                    widget.currentRow.IDNOMETAG.push(tag.IDNOMETAG);
                    widget.currentRow.DSNOMETAG.push(tag.DSNOMETAG);
                });
                widget.getField('IDNOMETAG').show();
            }
        }.bind(this));
    };
    
    this.getTags = function(code, parcOrTicket){
        var column
        if(parcOrTicket === 'PARCNEGOCIO'){
            column = "NRPARCNEGOCIO";
        }else if(parcOrTicket === 'CONTATO'){
            column = "NRSEQCONTATO"
        }
        var filter = [
            {
                "name": column,
                "operator": "=",
                "value": code
            }
        ];
        return localStorageEngine.download("product#/crud/crm_tag_"+parcOrTicket.toLowerCase(), filter);
    };
    
    this.setPeopleOnCurrentRow = function(widget){
        var currentRow = widget.currentRow;
        this.getPeople(currentRow.NRSEQCONTATO).then(function(response){
            if(!Util.isEmptyOrBlank(response)){
                widget.currentRow.NRPARCNEGOCIORELAC = [];
                widget.currentRow.NMPARCNEGOCIORELAC = [];
                response.forEach(function(person){
                    widget.currentRow.NRPARCNEGOCIORELAC.push(person.NRPARCNEGOCIO);
                    widget.currentRow.NMPARCNEGOCIORELAC.push(person.NMPRINCIPALPARC);
                });
                widget.getField('NRPARCNEGOCIORELAC').show();
            }
        }.bind(this));
    };
    
    this.getPeople = function(code){
        var filter = [
            {
                "name": 'NRSEQCONTATO',
                "operator": "=",
                "value": code
            }
        ];
        return localStorageEngine.download("product#/crud/crm_parcrelaccontato", filter);
    };
    
    this.ticketOnClick= function(args, e){
        var currentRow = e.item;
        var widget = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
        widget.currentRow = currentRow;
        ScreenService.openSwipe(widget).then(function(){
            this.setTagsOnCurrentRow(widget, 'CONTATO');
            this.setPeopleOnCurrentRow(widget);
            widget.templateToForm = "widget/master_detail/tabbedWidget.html";
            widget.templateToView = "widget/master_detail/tabbedWidget.html";
            widget.childTemplate = "widget/form-view.html";
            widget.computeDescriptionOrder();
            $('.swipe-parent-description').empty().append(widget.descriptionOrder);
            widget.view();
            widget.activate();
            this.controlActionsVisibilityByConclusao(widget);
            this.changeAnalistaReadOnly(widget);
        }.bind(this));
    };
    
    this.widgetCancelForm = function(widget){
        widget.childTemplate = "widget/form-view.html";
        General.widgetCancelForm(widget);
    };
    
    this.widgetEdit = function(widget){
        widget.childTemplate = "widget/form.html";
        General.widgetEdit(widget);
    };
    
    this.kanbanNewRow = function(widget){
        ScreenService.openSwipe(widget).then(function(){
            widget.templateToForm = "widget/master_detail/tabbedWidget.html";
            widget.templateToView = "widget/master_detail/tabbedWidget.html";
            widget.childTemplate = "widget/form.html";
            this.widgetNewRow(widget);
            this.controlActionsVisibilityByConclusao(widget);
        }.bind(this));    
    };
    
    this.buildRowsTransferir = function(widget){
        var tickets = widget.getCheckedRows();
        widget.currentRow = tickets;
        this.transferirChamado(widget);
    }
    
    this.rebuildKanban = function (widget) {
		if (widget.dataSource.data) {
			widget.columns.forEach(function(column) {
				column.items = widget.dataSource.data.filter(function(row) {
					return row.column == column.id;
				});
			});
		}
		//Espera o resize acabar e muda o tamanho do container
		var resizeTimer;
        $(window).on('resize', function(e) {
            clearTimeout(resizeTimer);
            resizeTimer = setTimeout(function() {
                this.fixContainerSize(widget);
            }.bind(this), 500);
        }.bind(this));
        
        $(window).resize();
	}
    
    this.openPopupNovaTag = function(widget){
        var wdgtTag = templateManager.container.getWidget('WDGT_NOVA_TAG');
        wdgtTag.newRow();
        wdgtTag.NAMEWDGT = widget
        ScreenService.openPopup(wdgtTag);
    }
    
    this.saveNovaTag = function(widget, widgetName){
        if(!widgetName)
            widgetName = widget.NAMEWDGT;
        var widPrincipal = templateManager.container.getWidget(widgetName);
        widget.currentRow.__using_unit =  false;
		widget.currentRow.__new_code_pk = widget.paramsNewCode.__new_code_pk;
		widget.currentRow.__table_pk = widget.paramsNewCode.__table_pk;
		widget.currentRow.__new_code_length = widget.paramsNewCode.__new_code_length || 5;
		widget.currentRow.__cast_number = false;
		widget.currentRow.__columns_to_return = widget.paramsNewCode.__columns_to_return;
		widget.saveRow().then(function(response){
		    ScreenService.closePopup();
		    if(!widPrincipal.currentRow.IDNOMETAG){
		        widPrincipal.currentRow.IDNOMETAG = [];
		        widPrincipal.currentRow.DSNOMETAG = [];
		    }
		    widPrincipal.currentRow.IDNOMETAG.push(response.dataset.columnsToReturn[0].IDNOMETAG);
		    widPrincipal.currentRow.DSNOMETAG.push(response.dataset.columnsToReturn[0].DSNOMETAG);
		    widPrincipal.getField('IDNOMETAG').show();
		    var selectWdgt = templateManager.container.getActiveWidget();
		    selectWdgt.clearSearchCriteria();
		    selectWdgt.reload().then(function(){
		       var novaTag = selectWdgt.dataSource.data.filter(function(tag){if(tag.IDNOMETAG === parseInt(response.dataset.columnsToReturn[0].IDNOMETAG)) return tag}); 
		       novaTag[0].__isSelected = true;
		       selectWdgt.updateCheckedRows();
		    });
		}).catch(function(err) {
		    ScreenService.showMessage(err.data.error);
		});
    };
    
    this.addInterlocutor = function(widget){
        if(Util.isEmptyOrBlank(widget.currentRow.NRPARCNEGOCIO)){
            ScreenService.showMessage(Messages.IMPOSSIVEL_MANUSEAR_EVENTO('Favor selecionar um cliente para cadastrar um novo interlocutor.')).then(function(){
                widget.currentRow.NRPARCINTERLOCUTOR = '';
                widget.currentRow.NMPARCINTERLOCUTOR = '';
                widget.currentRow.EMAIL = '';
                ScreenService.closePopup();
                setTimeout(function(){
                    $('.zh-field-NMPARCNEGOCIO').click();
                    widget.CREATINGINTERLOC = true;
                },4);
            });
        }else{
            var wdgtInter = templateManager.container.getWidget('WDGT_NOVO_INTERLOC');
            wdgtInter.newRow();
            wdgtInter.currentRow.NRPARCNEGOCIO = widget.currentRow.NRPARCNEGOCIO;
            ScreenService.openPopup(wdgtInter);
        }
    };
    
    this.afterSelectClient = function(widget){
        var tckWdgt = templateManager.container.getWidget('WDGT_CHAMADO');
        var kanWdgt = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
        var mainWdgt = Util.isEmptyOrBlank(tckWdgt.currentRow.NRPARCNEGOCIO) ? kanWdgt : tckWdgt; 
        if(mainWdgt.CREATINGINTERLOC){
            setTimeout(function(){
                var wdgtInter = templateManager.container.getWidget('WDGT_NOVO_INTERLOC');
                wdgtInter.newRow();
                wdgtInter.currentRow.NRPARCNEGOCIO = mainWdgt.currentRow.NRPARCNEGOCIO;
                ScreenService.openPopup(wdgtInter);
                mainWdgt.CREATINGINTERLOC = false;
            },4);    
        }
    };
    
    this.saveInterlocutor = function(widget){
        var tckWdgt = templateManager.container.getWidget('WDGT_CHAMADO');
        var kanWdgt = templateManager.container.getWidget('WDGT_CHAMADO_KANBAN');
        var mainWdgt = Util.isEmptyOrBlank(tckWdgt.currentRow.NRPARCNEGOCIO) ? kanWdgt : tckWdgt; 
        
        var paramsRestEngine = {
            "requestType": "Row",
            "row": widget.currentRow
        };
        widget.currentRow.NRCPFINTERLOC = widget.currentRow.NRCPFINTERLOC.replace(/[^0-9]/g, '');
        restEngine.post('/createInterloc', paramsRestEngine).then(function(response){
            if(response.dataset.INTERLOCUTOR){
                
                ScreenService.showMessage('Interlocutor criado com sucesso.');
                ScreenService.closePopup();
                
                mainWdgt.currentRow.NRPARCINTERLOCUTOR = response.dataset.INTERLOCUTOR.NRPARCNEGOCIO;
                mainWdgt.currentRow.NMPARCINTERLOCUTOR = response.dataset.INTERLOCUTOR.NMINTERLOCUTOR;
                mainWdgt.currentRow.EMAIL              = response.dataset.INTERLOCUTOR.EMAILINTERLOC;
                
            }else{
                ScreenService.showMessage('Falha ao Criar o Interlocutor.');    
            }
        }).
        catch(function(fail){
            ScreenService.showMessage('Falha ao Criar o Interlocutor.');
        }.bind(this));    
    };
    
    /**
     * Cancela o chamado
     * 
     * @param {object} widget widget de chamado
    */
    this.cancelarTicket = function(widget){
        var paramsRestEngine = {
            "requestType": "DataSet",
            "dataset": [widget.currentRow]
        };
        
        restEngine.post('/cancelTicket', paramsRestEngine).  
            then(function(response){
                if(response.dataset){
                    ScreenService.showMessage('Ticket Cancelado!');
                    ScreenService.closePopup(true);
                    ScreenService.closeSwipe().then(function(responsea){
                        widget.reload();
                        if(widget.name === "WDGT_CHAMADO_KANBAN")
                            this.setSituacaoOnKanbanColumns(widget);
                    }.bind(this));
                }
            }.bind(this)).
            catch(function(fail){
                ScreenService.showMessage('Falha ao Cancelar o Ticket.');
            }.bind(this));
    };
    
    /**
     * Corrige erro de paginação nos fiedls tipo select
    */
    this.setPaginationTreeGrid = function(field){
        field.hasPagination = false;    
    };
    
    this.loadwidgettree = function(args){
        console.log(args);
    }
}
Configuration(function(ContextRegister){
    ContextRegister.register('ChamadoController', ChamadoController);
});