function OrgOperController(templateManager, ScreenService, Utilities, restEngine, General, localStorageEngine, CRM_Utilities, Messages) {
    
    var changeActualOrg = false;
    
    this.saveOrganization = function() {
        var widget = templateManager.container.getActiveWidget();
        
        if(widget.isValid()) {
            widget.currentRow.NRORG = getLocalVar('CRM_NRORG');
            var paramsRestEngine = {
                "requestType": "Row",
                "row": widget.currentRow
            };
            
            restEngine.post('/saveOrganization', paramsRestEngine).then(function(response) {
                 if((response.dataset && response.dataset.error) || Util.isEmptyOrBlank(response)) {
                    ScreenService.showMessage(response.dataset.error[0], "ERROR");
                } else if (response.dataset) {
                    ScreenService.successNotification("Organização salva com sucesso!", 10000);
                    widget.reload().then(() => {
                        widget.view();
                    });
                    widget.currentRow.__is_new = false;
                    this.widgetOnSave(response.dataset.DADOS.CRM_NRORGTRAB);
                    General.changeWidgetAcessibility(widget.widgets, true);
                }
            }.bind(this)).catch(function(e) {
                var error;
                if(e.data.error === undefined) {
                    error = e;
                } else {
                    error = e.data.error;
                }
                ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error), 'ERROR');
            });
        }
    };
    
    this.deleteOrganization = function() {
        var widget = templateManager.container.getActiveWidget();
        
        if(widget.currentRow.CRM_NRORGTRAB === widget.currentRow.NRORG) {
            ScreenService.showMessage("Não é possível excluir a organização padrão!");
            return;
        }
        
        ScreenService.confirmMessage("Deseja mesmo excluir esta organização?", 'QUESTION',
        function(success) {
            General.callRestEngine("product#/deleteOrganization", "Row", widget.currentRow).then(function(data) {
                if (data.notifications &&data.notifications[0].type === "success") {
                    ScreenService.successNotification(Messages.REMOVE_ROW_SUCCESS, 10000);
                    Utilities.masterDetailVoltarList(widget);
                    widget.reload();
                } else if(data.messages) {
                    ScreenService.showMessage(data.messages[0].message, data.messages[0].type);
                    widget.currentRow.IDATIVO = 'N';
                    widget.reload();
                } else {
                    ScreenService.showMessage(Messages.REMOVE_ROW_ERROR);
                }
            }).catch(function(e) {
                var error;
                if(e.data.error === undefined) {
                    error = e;
                }else {
                    error = e.data.error;
                }
                ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error), 'ERROR');
            });
        });
        
    };
    
    this.saveOperators = function() {
        var widget = templateManager.container.getActiveWidget();
        if(widget.isValid()) {
            var nrorgtrab = widget.getParent().currentRow.CRM_NRORGTRAB;
            var rows = [];
            widget.currentRow.CDOPERADOR.forEach(function(operador, index) {
                if(widget.currentRow.CDOPERADOR[index] == templateManager.LOGGEDOPERATOR.CDOPERADOR) {
                    changeActualOrg = true;
                }
                rows.push(this.setOperatorsInRow(widget, index));
            }.bind(this));
            
            General.callRestEngine('product#/saveRelatedOperators', "DataSet", rows).then(function(response) {
                 if((response.dataset && response.dataset.error) || Util.isEmptyOrBlank(response)) {
                    ScreenService.showMessage(response.dataset.error[0], "ERROR");
                } else if (response.dataset) {
                    ScreenService.successNotification("Operador(es) salvo(s) com sucesso!", 10000);
                    ScreenService.closeSwipe();
                    widget.list();
                    widget.reload().then(() => {
                        ScreenService.confirmMessage(`Deseja alterar a organização de trabalho dos operadores adicionados para ${nrorgtrab}?`, 'CONFIRMATION', function() {
                            var dados = [
                                response.dataset.DADOS.CDOPERADOR,
                                widget.getParent().currentRow.CRM_NRORGTRAB
                            ];
                            General.callRestEngine('product#/alterarNrOrgTrab', "Row", dados).then((retorno) => {
                                ScreenService.successNotification("Organização de trabalho dos operadores modificada com sucesso!", 10000);
                                if(changeActualOrg) {
                                    setLocalVar("CRM_NRORGTRABALHO", nrorgtrab);
                                    var userInfo = widget.getParent().currentRow.NMORGTRAB + ' - ' + nrorgtrab;
                                    ScreenService.setHeaderInfo(userInfo);
                                    ScreenService.buildUserData(templateManager.LOGGEDOPERATOR.NMOPERADOR, [userInfo], function() {}, './assets/images/user.jpg');
                                    changeActualOrg = false;
                                    
                                    Object.keys(templateManager.containers).forEach(function(container, index){
                                        if(container !== "zeedhi_project" && container !== "product#dashboard" && container !== "product#par3001_orgoper") {
                                            templateManager.historyItems.forEach((item, index) => {
                                                if(item == container) {
                                                    templateManager.historyItems.splice(index, 1);
                                                }
                                    	    });
                                            delete templateManager.containers[container];
                                        }
                                    });
                                }
                            });
                        });
                    });
                } else {
                    ScreenService.showMessage("Erro ao tentar salvar o registro!<br>Por favor, contate um administrador!", "ERROR");
                }
            }.bind(this)).catch(function(e) {
                var error;
                if(e.data.error === undefined) {
                    error = e;
                }else {
                    error = e.data.error;
                }
                ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error), 'ERROR');
            });
        }
    };
    
    this.setOperatorsInRow = function(widget, index){
        var currentRow = angular.copy(widget.currentRow);

        currentRow.__is_new = true;
        currentRow.CDOPERADOR = currentRow.CDOPERADOR[index];
        currentRow.NMOPERADOR = currentRow.NMOPERADOR[index];
        currentRow.NRPARCNEGOCIO = currentRow.NRPARCNEGOCIO[index];
        currentRow.CRM_NRORGTRAB = widget.getParent().currentRow.CRM_NRORGTRAB;
        currentRow.NRORG = widget.getParent().currentRow.NRORG;

        return currentRow;
    };
    

    this.deleteOperators = function() {
        ScreenService.confirmMessage('Deseja realmente excluir os operadores selecionados?', 'CONFIRMATION', function(){
            var widget = templateManager.container.getActiveWidget();
            var params;
            var multipleTargets = false;
            
            if(!Util.isEmptyOrBlank(widget.getCheckedRows())) {
                params = widget.getCheckedRows();
                multipleTargets = true;
            } else {
                params = widget.currentRow;
                multipleTargets = false;
            }
            restEngine.post('product#/deleteOperators', {'requestType':'DataSet','dataset':[params]}).
                then(function(data){
                    if(data.messages){
                        if(data.messages[0].type == 'W') {
                            ScreenService.showMessage(data.messages[0].message, 'ERROR');
                        } else {
                            ScreenService.successNotification(Messages.REMOVE_ROW_SUCCESS + '! Os operadores que possuíam esta organização como a organização de trabalho ativa foram atualizados para a organização de trabalho matriz.', 10000);
                            if(multipleTargets) {
                                widget.unSelectAll();
                            } else {
                                ScreenService.closeSwipe();
                            }
                        }
                        widget.reload();
                    } else {
                        ScreenService.showMessage(Messages.REMOVE_ROW_ERROR);
                    }
                }).catch(function(e) {
                    var error;
                    if(e.data.error === undefined) {
                        error = e;
                    }else {
                        error = e.data.error;
                    }
                    ScreenService.showMessage(Messages.CATCH_EXCEPTION_ERROR(error), 'ERROR');
                });
            }
        );
    };
    
    this.setOperatorFilter = function() {
        var widget = templateManager.container.getActiveWidget();
        widget.dataSourceFilter[1].value = widget.getParent().currentRow.CRM_NRORGTRAB;
        widget.reload();
        
        widget.getField('CDOPERADOR').dataSourceFilter.find(x => x.name == 'P_CRM_NRORGTRAB').value = widget.getParent().currentRow.CRM_NRORGTRAB;
    };
    
    this.widgetOnEnterOnce = function(widget, fieldsToApply) {
        General.callRestEngine("product#/verifyMainOrg", "DataSet", []);
        
        var filterRow = {};
        filterRow.IDATIVO = 'S';
        
        CRM_Utilities.openFilterOnEnter(widget, filterRow, fieldsToApply);
    };
    
    this.goBackToMainGrid = function() {
        var mainWidget = templateManager.container.getWidget('gridOrgOper');
        
        ScreenService.closeSwipe();
        mainWidget.activate();
        mainWidget.list();
    };
    
    this.updateOrgTrabalho = function() {
        var widget = templateManager.container.getActiveWidget();
        var nrorgtrab = widget.getParent().currentRow.CRM_NRORGTRAB;
        var rows = widget.getCheckedRows();
        var cdoperador = [];
        rows.forEach(function(row, index) {
            cdoperador.push(row.CDOPERADOR);
        });
        var params = [cdoperador, nrorgtrab];

        ScreenService.confirmMessage(`Deseja alterar a organização de trabalho dos operadores selecionados para ${nrorgtrab}?`, 'CONFIRMATION', function() {
            General.callRestEngine('product#/alterarNrOrgTrab', "Row", params).then(() => {
                ScreenService.successNotification("Organização de trabalho dos operadores modificada com sucesso!", 10000);
            });
        });
    };
	
    this.widgetOnSave = function(crm_nrorgtrab) {
        var widget = templateManager.container.getActiveWidget();
        var filter = [
            {
                "name": "CRM_NRORGTRAB",
                "operator": "=",
                "value": crm_nrorgtrab
            },
            {
                "name": "P_NRORG",
                "operator": "=",
                "value": widget.currentRow.NRORG
            }
        ];
        General.callRestEngine("product#/findOrganization", "FilterData", filter).then((response) => {
            widget.currentRow.CRM_NRORGTRAB = response.dataset.orgoper[0].CRM_NRORGTRAB;
            widget.currentRow.NMORGTRAB = response.dataset.orgoper[0].NMORGTRAB;
            widget.currentRow.IDATIVO = response.dataset.orgoper[0].IDATIVO;
            widget.currentRow.DSORG = response.dataset.orgoper[0].DSORG;
            widget.currentRow.DTINCLUSAO = response.dataset.orgoper[0].DTINCLUSAO;
            widget.currentRow.NMOPERINCLUSAO = response.dataset.orgoper[0].NMOPERINCLUSAO;
            widget.currentRow.DTULTATU = response.dataset.orgoper[0].DTULTATU;
            widget.currentRow.NMOPERULTATU = response.dataset.orgoper[0].NMOPERULTATU;
        });
    };
}

Configuration(function(ContextRegister) {
    ContextRegister.register('OrgOperController', OrgOperController);
});