/* global Util, UtilDevTools inject expect */
describe("DTRenderService", function(){

	var dtRenderService;
	var metaDataFactory;
	var templateManager;
	var DASHBOARD = "container/dashboard.html";
	var defaultContainer = {
		"name": "testContainer",
		"label": "test Container",
		"id": "01",
		"widgets":[
			{
				"name": "testWidget",
				"label": "test Widget",
				"id": "02"
			}]
	};

	var project = {
		"name": "testProject",
		"label": "test Project",
		"id": "001",
		"groupMenu":[{
			"id": "002",
			"name": "groupMenu01"
		}]
	};

	var devToolsWindowMock = {
		"groupMenu" : {
			containers:	{
				zeedhi_project: {},
				__metadata: {}
			},
			groupMenu: [Util.clone(project.groupMenu)]
		}
	};

	var windowController = function(){
		this.clearInvalidItems = function(){};
		this.updateActiveItemInEditor= function(){};
	};

	var hasHistoryMenu = function(project){
		var historyMenuName = project.historyMenuGroup;
		if(historyMenuName){
			var historyMenu = project.groupMenu.filter(function(menu){
				return menu.name == historyMenuName;
			})[0];
			historyMenu = historyMenu.menus.filter(function(menu){
					return menu.historyMenu;
			});
		}
		return historyMenu ? !!Object.keys(historyMenu).length : false;
	};

	function mockTemplateManagerContainer(){
		var container = Util.clone(defaultContainer);
		templateManager.container = metaDataFactory.containerFactory(container);
		templateManager.container.widgets[0] = metaDataFactory.widgetFactory(container.widgets[0], container);
		templateManager.container.__metadata = Util.clone(container);
		templateManager.container.__metadata.fileName = container.name + ".json";
		templateManager.container.__metadata.__indexPath = "./json/containers/" + container.name + ".json";
	}

	UtilDevTools.getMainWindow().getDevToolsWindow = function(){
		return devToolsWindowMock;
	};

	beforeEach( function(){
		window.templateUrl = "../../dyman/templates/dyman/";
		window.metadataUrl = "/json_test/test";
		window.serviceUrl = "../backend/service/index.php";
		module('ZeedhiApp');

		inject( function( _metaDataFactory_, _localStorageEngine_, _templateManager_, _ApplicationContext_){
			_ApplicationContext_.registerInstance("WindowController", new windowController());
			templateManager = _templateManager_;
			metaDataFactory = _metaDataFactory_;
			mockTemplateManagerContainer();
			dtRenderService = new DTRenderService(metaDataFactory, templateManager, _ApplicationContext_);
		});
	});

	it("UpdateProjectConfig when pass a project with id different from templateManager shouldn't update groupMenu.", function(){
		templateManager.project = Util.clone(project);
		var newProject = Util.clone(project);
		newProject.id = "123456";
		newProject.groupMenu[0].name = "groupHasMenuChanged";
		dtRenderService.updateProjectConfig(newProject);
		expect(templateManager.project.groupMenu[0].name).toBe(project.groupMenu[0].name);
	});

	it("UpdateProjectConfig when pass a project with id is equal of the templateManager should update groupMenu.", function(){
		templateManager.containers = {};
		templateManager.containers.zeedhi_project = templateManager.project = Util.clone(project);
		var newProject = Util.clone(project);
		newProject.groupMenu[0].name = "groupMenuChanged";
		dtRenderService.updateProjectConfig(newProject);
		expect(hasHistoryMenu(templateManager.project)).toBe(false);
		expect(devToolsWindowMock.groupMenu.containers.zeedhi_project.groupMenu[0].name).toBe(newProject.groupMenu[0].name);
		expect(templateManager.project.groupMenu[0].name).toBe(newProject.groupMenu[0].name);
	});

	it("RenderContainer when container sended has a change should change container of templateManager.", function(){
		var newContainer = Util.clone(defaultContainer);
		var newContainerLabel = "new Container";
		newContainer.label = newContainerLabel;
		dtRenderService.renderContainer(newContainer);
		expect(templateManager.container.label).toBe(newContainerLabel);
		expect(templateManager.container.__metadata.label).toBe(newContainerLabel);
		expect(templateManager.container.showInDevTools).toBe(true);
	});

	it("RenderContainer when container sended has dashboard as template should change isVisited of container of templateManager.", function(){
		var newContainer = Util.clone(defaultContainer);
		newContainer.template = DASHBOARD;
		dtRenderService.renderContainer(newContainer);
		expect(templateManager.container.template).toBe(DASHBOARD);
		expect(templateManager.container.__metadata.template).toBe(DASHBOARD);
		expect(templateManager.container.__isVisited).toBe(true);
	});

	it("RenderContainer when templateManager.container has onRender and dashboard as template should call onRender.", function(){
		templateManager.container.onRender = function(){ templateManager.container.onRenderisCalled = true;};
		var newContainer = Util.clone(defaultContainer);
		newContainer.template = DASHBOARD;
		dtRenderService.renderContainer(newContainer);
		expect(templateManager.container.template).toBe(DASHBOARD);
		expect(templateManager.container.__metadata.template).toBe(DASHBOARD);
		expect(templateManager.container.__isVisited).toBe(false);
		expect(Util.isFunction(templateManager.container.onRender)).toBe(true);
		expect(templateManager.container.onRenderisCalled).toBe(true);
	});
});