/* global  angular, Util, UtilDevTools, ShowInDevTools, $ */
var ZeedhiDirectives = angular.module("ZeedhiDirectives");

ZeedhiDirectives.directive("zhComponentsTree", function($timeout, templateManager, ElementToMetadataService, eventAggregator, ElementFileService, DevToolsService, DeveloperTreeService) {
	return {
		restrict: 'A',
		link: function($scope, $element, $attributes, $controller) {
			$scope.lastScript = "";
			var ACTIVE_ITEM_CLASS = "active-item";
			var ERROR_ITEM_CLASS = "error-validation";
			var FILE_TYPE = "files";
			var JSON_TYPE = "json";
			var MENU_TYPE = "menu";
			var sidebar = $element;
			var invalidItems = [];
			var unloadedInvalidItems = [];
			var DevToolsWindow = window;
			var subitemsPropertiesMap = UtilDevTools.getZeedhiTypesSubitems();

			function getItemMetadataByPath(path) {
				var container = DeveloperTreeService.getTreeItems($scope.getCurrentMode());
				return ElementToMetadataService.getJsonElement(path, container[0]);
			}
			$scope.getItemMetadataByPath = getItemMetadataByPath;

			function getParent(element, elementType, position){
				var classElement = ".tree-item." + elementType;
				var child = $(element.children("ul"));
				var parents = child.find('span' + classElement);
				parents = parents.length ? parents.parent() : child.children("li");
				return $(parents[position]);
			}

			function setTreeItemAsSelected(item){
				var element = item._element;
				if(element){
					var unRegister = $scope.$watch(function(){
						return element.hasClass(getItemType(item)) || element.hasClass(getFileSystemType(item));
					}, function(){
						if(element.hasClass(getItemType(item)) || element.hasClass(getFileSystemType(item))){
							$('ul#containers-list').find('.' + ACTIVE_ITEM_CLASS).removeClass(ACTIVE_ITEM_CLASS);
							element.addClass(ACTIVE_ITEM_CLASS);
							unRegister();
						}
					});
				}
			}

			var loadInvalidItems = Util.buildDebounceMethod(function(){
				unloadedInvalidItems.forEach(function(invalidItem){
					addInvalidItem(invalidItem);
				});
				unloadedInvalidItems = [];
			}, 200);

			function setActiveModeTab(){
				$('.tabs').find('.active').removeClass('active');
				$element.find("#" + $scope.currentMode).addClass('active');
				if($scope.inJsonMode()){
					loadInvalidItems();
				} else if($scope.inFileMode()){
					unloadedInvalidItems = invalidItems;
					invalidItems = [];
				}
			}

			var removeExtraProperties = UtilDevTools.buildInCustomizationModeMethod(function removeExtraProperties(json){
				var zeedhiTypes = UtilDevTools.getZeedhiTypes().concat(UtilDevTools.getIgnoredZeedhiTypes());
				zeedhiTypes.forEach(function deleteSubitems(type){
					delete json[type];
				});
				delete json.id;
			});

			function clearJSONAndStringify(json){
				if(json){
					removeExtraProperties(json);
					delete json.__indexPath;
					delete json.fileName;
					var cleanJSON = ShowInDevTools.clear(json);
					return JSON.stringify(cleanJSON, null, "\t");
				}
			}

			function getAnnotationMessage(element) {
				element = Array.isArray(element) && element[0] ? element[0] : null;
				var message = element ? element.scope().errorMessage : "";
				if(message){
					message = {
						"row": 0,
						"text": message,
						"type": "warning"
					};
				}
				return message;
			}

			function getItemText(itemMetadata){
				var text = $scope.inFileMode() ? itemMetadata.fileElement : itemMetadata.jsonElement;
				return $scope.inFileMode() ? text : clearJSONAndStringify(Util.clone(text));
			}
			$scope.getItemText = getItemText;

			function isSamePath(activePath, newPath){
				var hasActivePathInNewPath = newPath.indexOf(activePath) >= 0;
				return !activePath || hasActivePathInNewPath;
			}

			function getProperties(item){
				var type = getItemType(item);
				var properties = type ? subitemsPropertiesMap[type] : [];
				if(!type){
					var files = [];
					var folders = [];
					var exceptions = ["parent", "parentMenu", "$$hashKey", "_element", "_opened", "__metadata"];
					Object.keys(item).forEach(function(key){
						if(Util.isObject(item[key]) && item[key].isFolder){
							folders.push(key);
						} else if(exceptions.indexOf(key) < 0){
							files.push(key);
						}
					});
					properties = folders.concat(files);
				}
				return properties;
			}

			function getElement(path, parent){
				parent = parent ? parent : $(".tree-item.container").parent();
				var parentElement = parent.children('span');
				if (path.length >= 2) {
					path = Util.clone(path);
					var elementType = path.shift().slice(0, -1);
					var position = path.shift();
					var newParent = getParent(parent, elementType, position);
					parentElement = getElement(path, newParent);
				}
				return parentElement;
			}

			function openLastPath(){
				var lastItem = DeveloperTreeService.getLastItem($scope.getCurrentMode());
				if(lastItem && lastItem.path){
					$scope.openPath(lastItem.path);
				}
			}

			function validModes(newMode, oldMode){
				var modeChange = newMode != oldMode;
				var hasModes = newMode && oldMode;
				var fromForm = (newMode == 'json' && oldMode == 'form');
				return hasModes && modeChange && !fromForm;
			}

			function forceModeFile(activeMode){
				return (activeMode == 'css' || activeMode == 'html') ? 'files' : activeMode;
			}

			function getTextAndOpenLastPath(text, items){
				var newMode = forceModeFile(items.newItem.mode);
				var oldMode = forceModeFile(items.oldItem.mode);
				if(validModes(newMode, oldMode)){
					openLastPath();
					text = "";
				}
				return text;
			}
			$scope.getTextAndOpenLastPath = getTextAndOpenLastPath;

			function unsetElementOfTree(obj){
				if(typeof obj == "object"){
					delete obj._element;
					getProperties(obj).forEach(function(property){
						if(Array.isArray(obj[property])){
							obj[property].forEach(function(child){
								unsetElementOfTree(child);
							});
						} else if(typeof obj[property] == "object"){
							unsetElementOfTree(obj[property]);
						}
					});
				}
			}

			function itemHasChildren(item) {
				return (getSubitems(item).length > 0);
			}
			$scope.itemHasChildren = itemHasChildren;

			function replaceElementOfActiveItem(){
				var item = $scope.activeItem.item;
				var needReplace = $scope.activeItem.cursorPosition || item._element && item._element.is(":hidden");
				if(!item.isFolder && item._element && needReplace){
					var element = getElement($scope.activeItem.path);
					item._element = element.length > 0 ? element : item._element;
				}
			}

			function getModeClass(path){
				return path.substring(path.lastIndexOf(".") + 1, path.length);
			}

			function updateClassMode(indexPath){
				var classMode = getModeClass(indexPath);
				classMode = classMode == "js" ? 'javascript' : classMode;
				$scope.currentModeClass = ace.require("ace/mode/".concat(classMode)).Mode;
			}

			function setActiveItemObject(item, itemMetadata, cursorPosition){
				var path = itemMetadata.path;
				var text = getItemText(itemMetadata);
				var indexPath = DeveloperTreeService.getIndexPath($scope.getCurrentMode(), itemMetadata);
				var annotationMessage = getAnnotationMessage(item._element);
				$scope.lastScript = text;
				$scope.activeItem = {
					"item": item,
					"text": text,
					"path": path,
					"errorMessage": annotationMessage,
					"cursorPosition": cursorPosition,
					"_indexPath": indexPath
				};
				updateClassMode(indexPath);
				var itemParent = !$scope.inFileMode() ? DevToolsWindow.currentContainer.name : "";
				DeveloperTreeService.setActiveItem($scope.activeItem, $scope.getCurrentMode(), itemParent);
				replaceElementOfActiveItem();
				setTreeItemAsSelected(item);
			}

			function setActiveFolder(item){
				var activeFolder = false;
				if(item.isFolder){
					var pathSeparator = '/';
					var path = item.url.split(pathSeparator);
					var severalPaths = item.url.indexOf(pathSeparator) >= 0;
					if(severalPaths){
						path.shift();
					}
					activeFolder = {
						name: item.name,
						path: path,
						mode: FILE_TYPE,
						parent: ''
					};
					DeveloperTreeService.setLastItem(FILE_TYPE, activeFolder);
				}
				$scope.activeFolder = activeFolder;
			}

			function setActiveItem(item, itemMetadata, cursorPosition){
				setActiveFolder(item);
				if(!item.isFolder){
					itemMetadata = itemMetadata || DeveloperTreeService.getItemMetadata(item, $scope.getCurrentMode());
					if(itemMetadata){
						if(itemMetadata.then){
							itemMetadata.then(function(obj){
								setActiveItemObject(item, obj, cursorPosition);
							});
						} else if(itemMetadata.path){
							setActiveItemObject(item, itemMetadata, cursorPosition);
						}
					}
				}
			}
			$scope.setActiveItem = setActiveItem;

			function updateActiveItem(paths){
				var activeItem = getActiveItem();
				if(activeItem){
					var activePath = activeItem.path.join("/");
					paths.some(function(path){
						if(isSamePath(activePath, path)){
							var activeItemMetadata = getItemMetadataByPath(activeItem.path);
							var cursorPosition = $scope.currentEditor.getCursorPosition();
							setActiveItem(activeItem.item, activeItemMetadata, cursorPosition);
							return true;
						} else {
							return false;
						}
					});
				}
			}
			$scope.updateActiveItem = updateActiveItem;

			function getActiveItem() {
				return $scope.activeItem;
			}
			$scope.getActiveItem = getActiveItem;

			function getItemType(item) {
				return ElementToMetadataService.getPropertyFromScope(item);
			}
			$scope.getItemType = getItemType;

			function getFileSystemType(item){
				var icon = "code-file";
				item = item || {};
				if(!item.isFile){
					icon = item._opened ? "open-folder" : "closed-folder";
				}
				return icon;
			}
			$scope.getFileSystemType = getFileSystemType;

			function getSubitems(item) {
				var subitems = [];
				if (item) {
					getProperties(item).forEach(function(property){
						if(item[property] && Array.isArray(item[property])){
							subitems = subitems.concat(item[property]);
						} else if(typeof item[property] == 'object'){
							subitems.push(item[property]);
						}
					});
				}
				return subitems.filter(function(subitem){
					return subitem.showInDevTools;
				});
			}
			$scope.getSubitems = getSubitems;

			function toggleItem(item, DevToolsController, cursorPosition) {
				if(item){
					if (itemHasChildren(item)) {
						item._opened = !item._opened;
						templateManager.updateTemplate();
					}
					if(!item.isFolder){
						if(DevToolsController && $scope.textIsReady){
							DevToolsController.saveComponentChange();
						}
						$scope.textIsReady = false;
					}
					setActiveItem(item, false, cursorPosition);
				}
			}
			$scope.toggleItem = toggleItem;

			function getItems(){
				return DeveloperTreeService.getTreeItems($scope.getCurrentMode());
			}
			$scope.getItems = getItems;

			function restoreJSONTreeToDefaultState(tree) {
				if(!$scope.inFileMode || !$scope.inFileMode()){
					$scope.activeItem = undefined;
					DevToolsService.clearActiveItem();
					$scope.closeSubtree(tree);
					sidebar.find("." + ACTIVE_ITEM_CLASS).removeClass(ACTIVE_ITEM_CLASS);
				}
				DeveloperTreeService.setLastItem(JSON_TYPE, undefined);
			}

			function addInvalidItem(errorObject) {
				var path = Util.clone(errorObject.path);
				var element = getElement(path);
				if($scope.inFileMode() || !element){
					unloadedInvalidItems.push(errorObject);
				} else {
					var invalidItem = {
						path: path,
						element: element,
						errorMessage: errorObject.errorMessage
					};
					invalidItems.push(invalidItem);
					element.addClass(ERROR_ITEM_CLASS);
					if(element.scope()){
						element.scope().errorMessage = invalidItem.errorMessage;
					}
					if(element.hasClass(ACTIVE_ITEM_CLASS)){
						var annotation = getAnnotationMessage(element);
						$scope.setAnnotations([annotation]);
					}
				}
			}
			$scope.addInvalidItem = addInvalidItem;

			function clearInvalidItems() {
				invalidItems.forEach(function(item){
					item.element.removeClass(ERROR_ITEM_CLASS);
					if(item.element.scope()){
						item.element.scope().errorMessage = "";
					}
				});
				invalidItems = [];
				$scope.setAnnotations([]);
				requestAnimationFrame(function(){});
			}
			$scope.clearInvalidItems = clearInvalidItems;

			$scope.setInvalidItems = function(errorsObject) {
				clearInvalidItems();
				errorsObject.forEach(function(object){
					addInvalidItem(object);
				});
			};

			$scope.reloadErrorsValidationInTree = Util.buildDebounceMethod(function() {
				invalidItems.forEach(function(item){
					item.element.addClass(ERROR_ITEM_CLASS);
				});
			}, 200);

			$scope.closeSubtree = function(item) {
				var subitems = getSubitems(item);
				subitems.forEach(function(child) {
					$scope.closeSubtree(child);
				});
				if (itemHasChildren(item)) {
					item._opened = false;
				}
				templateManager.updateTemplate();
			};

			$scope.openPath = function(pathToOpen, DevToolsController, cursorPosition){
				DeveloperTreeService.loadEmptyTrees(); // @toDo colocar isso num lugar que execute menos
				var currentMode = $scope.getCurrentMode();
				var lastItem = DeveloperTreeService.getTreeItems(currentMode)[0];
				$scope.closeSubtree(lastItem);
				if(pathToOpen && pathToOpen.length >= 0 && lastItem){
					if(pathToOpen.length === 0){
						lastItem._opened = true;
					} else {
						lastItem = DeveloperTreeService.openTree(currentMode, pathToOpen, lastItem);
					}
					toggleItem(lastItem, DevToolsController, cursorPosition);
				}
			};

			$scope.getActiveComponentPath = function() {
				return $scope.activeItem ? $scope.activeItem.path : null;
			};

			$scope.clearLastItem = function(){
				DeveloperTreeService.clearLastItems();
			};

			function toggleSideMenu(){
				var isSideMenuOpen = UtilDevTools.getMainWindow().executeFunction('ScreenService', 'isSideMenuOpen');
				var openSideMenu = $scope.inMenuMode() && !isSideMenuOpen;
				var closeSideMenu = $scope.lastMode == 'menu' && isSideMenuOpen;
				if(openSideMenu || closeSideMenu){
					UtilDevTools.getMainWindow().executeFunction('ScreenService', 'toggleSideMenu');
				}
			}

			$scope.$watch("currentMode", function(){
				var formModeIsNotRelated = $scope.lastMode && $scope.lastMode != 'form' && !$scope.inFormMode();
				if(formModeIsNotRelated){
					unsetElementOfTree(DeveloperTreeService.getTreeItems($scope.lastMode));
				}
				if($scope.inFileMode()){
					DeveloperTreeService.updateTree(FILE_TYPE);
				}
				toggleSideMenu();
				DeveloperTreeService.restoreUnloadedItems();
			});

			function initMode(){
				var currentMode = $scope.getCurrentMode();
				var activeItem = DevToolsService.getLocalActiveItem() || {};
				var lastActiveItem = DeveloperTreeService.getLastItem(currentMode);
				var path = activeItem.mode == currentMode ? activeItem.path : [];
				path = path || [];
				if(lastActiveItem && Object.keys(lastActiveItem).length > 0){
					path = lastActiveItem.path || path;
				} else {
					$scope.activeItem = undefined;
				}
				$scope.textIsReady = false;
				$scope.openPath(path, $scope.DevToolsCtrl, activeItem.cursorPosition);
			}

			// Event emitted when the component-tree is completed render
			eventAggregator.subscribe("finishedRenderTreeItems", function(){
				initMode();
				setActiveModeTab();
			});

			// Backups current value because Object.defineProperty resets the value of the property
			var oldValue = window.currentContainer;
			var auxValue;
			Object.defineProperty(window, 'currentContainer', {
				set: function(newValue) {
					DeveloperTreeService.updateTree(JSON_TYPE, newValue);
					if (newValue) {
						restoreJSONTreeToDefaultState(newValue);
						if ($scope.initEditor) $scope.initEditor();
						if ($scope.clearEditor) $scope.clearEditor();
						$timeout(function() {
							$scope.$apply();
						}, 0, false);
						auxValue = newValue;
					}
				},
				get: function() {
					return auxValue;
				}
			});
			window.currentContainer = oldValue;
		}
	};
});