/* global Util, ShowInDevTools, diff, Widget, UtilDevTools */
function DTRenderService(metaDataFactory, templateManager, ApplicationContext){
    var DASHBOARD = "container/dashboard.html";
    var typeMap = {
        "fields": {
            "factoryMethod": metaDataFactory.fieldFactory
        },
        "widgets": {
            "factoryMethod": metaDataFactory.widgetFactory
        },
        "actions": {
            "factoryMethod": metaDataFactory.actionFactory
        }
    };

    var getWindowController = function(){
        return ApplicationContext.WindowController;
    };

    var factoryObject = function(type, obj, objParent){
        if(typeMap[type]){
            obj = typeMap[type].factoryMethod(obj, objParent);
        }
        return obj;
    };

    //Returns the parent, value and position of the object modified by that path
    var getObjDetails = function(path, obj){
        var result = {};
        if(path.length == 2){
            var type = path[0];
            var position = path[1];
            result.parent = obj;
            result.position = position;
            result.objValue = obj[type][position];
        } else {
            var key = path.shift();
            var newObj = obj[key];
            result = getObjDetails(path, newObj);
        }
        return result;
    };

    var getLastComponentIndex = function(parts){
        var index;
        if(parts.lastIndexOf('fields') >= 0){
            index = parts.lastIndexOf('fields');
        } else if(parts.lastIndexOf('actions') >= 0){
            index = parts.lastIndexOf('actions');
        } else {
            index = parts.lastIndexOf('widgets');
        }
        return index;
    };

    var needToRefactory = function(lastIndexOfChangedComponent, option, parts){
        var result = false;
        var hasChanges = lastIndexOfChangedComponent !== -1;
        var isDeleted = option == 'delete';
        var componentWasDeleted = (isDeleted && lastIndexOfChangedComponent != parts.length - 2);
        result = hasChanges && (!isDeleted || componentWasDeleted);
        return result;
    };

    var getPartsToComponets = function(lastIndexOfChangedComponent, parts){
        return parts.slice(0, lastIndexOfChangedComponent + 2);
    };

    var replaceComponentAtPosition = function(parent, component, position){
        parent.splice(position, 1, component);
        return parent;
    };

    var updateContainerActions = function(){
        templateManager.container.actions = null;
        templateManager.container.prepareActions();
    };

    var updateContainer = function(path, option){
        var parts = path.split('/');
        parts.shift();
        var lastIndexOfChangedComponent = getLastComponentIndex(parts);
        var partsToComponent = getPartsToComponets(lastIndexOfChangedComponent, parts);
        var componentCollection = partsToComponent[lastIndexOfChangedComponent];
        if(needToRefactory(lastIndexOfChangedComponent, option, parts)){
            var preparedObj = getObjDetails(partsToComponent, templateManager.container);
            var parentToFactory = preparedObj.parent;
            var parentArray = parentToFactory[componentCollection] || [];

            var types = Object.keys(typeMap);
            if(types.indexOf(componentCollection) !== -1){
                if(componentCollection == 'widgets'){
                    parentToFactory = templateManager.container;
                }
                var obj = factoryObject(componentCollection, preparedObj.objValue, parentToFactory);
                replaceComponentAtPosition(parentArray, obj, preparedObj.position);
            }
        }
        if(componentCollection == 'actions'){
            updateContainerActions();
        }

        var needRenderContainer = templateManager.container.template == DASHBOARD;
        if(needRenderContainer){
            templateManager.container.__isVisited = true;
        }
    };

    var visitorDiffCallback = function(val){
        if(val instanceof Widget){
            val.__isVisited = true;
        }
    };

    var resetVisitedWidget = function(widget){
        if(widget.__isVisited && widget.onRender){
            widget.onRender();
        }
        widget.__isVisited = false;
    };

    var updateEditor = function(keysDiffs){
        getWindowController().clearInvalidItems();
        getWindowController().updateActiveItemInEditor(keysDiffs);
    };

    var updateTemplateManagerContainer = function(containerComming){
        var fileName = templateManager.container.__metadata.fileName;
        var indexPath = templateManager.container.__metadata.__indexPath;
        containerComming.fileName = fileName;
        containerComming.__indexPath = indexPath;
        ShowInDevTools.setTree(containerComming);
    };

    this.renderContainer = UtilDevTools.buildMainWindowMethod(function(newContainer){
        updateTemplateManagerContainer(newContainer);
        var diffs = diff.getDiff(templateManager.container.__metadata, newContainer);
        var keysDiffs = Object.keys(diffs);
        templateManager.container.__metadata = Util.clone(newContainer);
        diff.applyDiff(templateManager.container, diffs, visitorDiffCallback);
        keysDiffs.reverse().forEach(function(path){
            if(path != '/fileName'){
                var diffOp = diffs[path].operation;
                updateContainer(path, diffOp);
            }
        });
        updateEditor(keysDiffs);

        if(templateManager.container.__isVisited && templateManager.container.onRender){
            templateManager.container.onRender();
            templateManager.container.__isVisited = false;
        }
        templateManager.container.eachWidget(resetVisitedWidget);
    });

    var searchMenu = function(element, elementHistory){
        return element.groupMenu.find(function(menu){
            return menu.name == elementHistory;
        });
    };

    var insertHistoryMenuInProject = function(elementToInsert, element){
        var elementHistory = element.historyMenuGroup;
        var insertHistory = elementToInsert.historyMenuGroup;
        if(elementHistory && insertHistory){
            var historyMenu = searchMenu(element, elementHistory);
            historyMenu = historyMenu.menus.find(function(menu){
                return menu.historyMenu;
            });
            var menu = searchMenu(elementToInsert, insertHistory);
            menu.menus.push(historyMenu);
        }
    };

    var setPropertiesInProject = function(currentProject ,newProject , projectReceived){
        var indexPath = currentProject.__indexPath;
        newProject.__indexPath = indexPath;
        templateManager.project = metaDataFactory.projectFactory(newProject);
        templateManager.project.__indexPath = indexPath;
        templateManager.project.__menuMetadata = projectReceived.groupMenu;
        templateManager.updateTitle(currentProject.label);
        templateManager.updateTemplate();
    };

    function updateMetadataGroupMenu(devToolsWindow, newGroupMenu){
        var menuContainers = devToolsWindow.groupMenu.containers;
        menuContainers.zeedhi_project.groupMenu = Util.clone(newGroupMenu);
        menuContainers.__metadata.groupMenu = menuContainers.zeedhi_project.groupMenu;
    }

    function insertDiffsInMenu(zeedhi_project, newGroupMenu, devToolsWindow){
        var diffs = diff.getDiff(zeedhi_project.groupMenu, newGroupMenu);
        diff.applyDiff(devToolsWindow.groupMenu.groupMenu, diffs, visitorDiffCallback);
        var insertGroupMenu = Object.keys(diffs).map(function(value){
             return "groupMenu" + value;
        });
        updateEditor(insertGroupMenu);
    }

    function updateGroupMenu(newGroupMenu){
        var devToolsWindow = UtilDevTools.getMainWindow().getDevToolsWindow();
        var zeedhi_project = templateManager.containers.zeedhi_project;
        updateMetadataGroupMenu(devToolsWindow, newGroupMenu);
        insertDiffsInMenu(zeedhi_project, newGroupMenu, devToolsWindow);
        zeedhi_project.groupMenu = newGroupMenu;
    }

    this.updateProjectConfig = UtilDevTools.buildMainWindowMethod(function(projectChanged){
        var currentProject = templateManager.project;
        var newProject = Util.clone(projectChanged);
        if(newProject.id == currentProject.id){
            insertHistoryMenuInProject(newProject, currentProject);
            updateGroupMenu(projectChanged.groupMenu);
            setPropertiesInProject(currentProject, newProject, projectChanged);
        }
    });
}
